/***************************************************************************
                          configuration.h  -  description
                             -------------------
    begin                : Tue Feb 1 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk

    rewrite/rc reading   : Sun Jan 30 2000
    copyright            : (C) 2000 by Eero Tamminen
    email                : t150315@cc.tut.fi

    rerewrite            : Sun Feb 02 2000, dion@swamp.dk

 ***************************************************************************/


#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include "systemheaders.h"

//Used to return values on the form "123x456"
typedef struct Config_xy {
	int x;
	int y;
};

class Configuration_observer;
class Config_values;
class Config_observers;
class Configuration {
public: 
	Configuration();
	~Configuration();

	//use these to fill up the object:
	bool parse_cmdline(int argc,char **argv);
	bool parse_file(char *file_contents);
	bool load_file(char *filename);
  bool parse_option(char *line);

	//value access:
	bool clear_option(char *name);
	bool set_option(char *name, char *value);
	const char *get_option(char *name);

	//sugar coated value access:
	int get_option_int(char *name);
	Config_xy get_option_xy(char *name);
	
	//error reporting handling:
	char *get_error() {return error;}

	//lists all the options:
	void print_manual(FILE *target);

	//checks the highlevel constraints:
	bool check_options();

	bool save_options(char *filename);
	void print_options(FILE *target);

	//observer/subject stuff:
	void attach(Configuration_observer *observer);
	void detach(Configuration_observer *observer);

protected:
	Config_observers *observers;
	//This should be called internally to do what it says
	void notify_observers();
	
	Config_values *values;
	void add_option(char *name, char *description, char *default_value, int min=0, int max=0, bool integer_value=false, bool append_value=false);

	char error[200];
};

//inherit from this if you want to be notified when the config changes:
class Configuration_observer {
public:
	Configuration_observer();
	virtual ~Configuration_observer();
	void config_attach(Configuration *config);

	//impelement this to get notified by the Configuration when it changes
 	virtual void config_changed()=0;

protected:
  void detatch();
	
	//This is the configuration that we are watcing:
	Configuration *configuration;
};

#endif
