/***************************************************************************
               fluff.cpp  -  Misc routines for doing tedious work
                             -------------------
    begin                : Sat Oct 9 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/
#include "fluff.h"
#include "systemheaders.h"
#include "raw_surface.h"
#ifndef KILL_ON_ASSERTION
#include <assert.h>
#endif 

int *stomped_int_fuckedup = NULL;
int stomped_int_correct;

bool request_quit = false;

int rnd(int min, int max)
{
	if (min > max)
		return max + (random() % (1+min-max));
	else
		return min + (random() % (1+max-min));
}

unsigned int mkrndcolor(unsigned int c1, unsigned int c2)
{	
	int r = rnd(GETRED(c1)  ,GETRED(c2));
	int g = rnd(GETGREEN(c1),GETGREEN(c2));
	int b = rnd(GETBLUE(c1) ,GETBLUE(c2));

	int color = ((r & 0xff)<<16) + ((g & 0xff)<<8) + (b & 0xff);
	
	return color;
}

Logtype chattiness = lt_debug;

void setchattiness(Logtype new_chattiness)
{
	if (new_chattiness >= lt_fatal && new_chattiness <= lt_assert)
		chattiness = new_chattiness;
}

char msgsource[100];

void setmsgsource(char *source)
{
	strcpy(msgsource,source);
}

void logmsg(Logtype logtype, char *format, ...)
{
	if (logtype > chattiness && logtype != lt_assert) return;
	printf("%s: ",msgsource);
	switch (logtype) {
		case lt_assert 	: printf("assertion failed: "); break;
		case lt_debug 	: printf("debug: "); break;
		case lt_message : printf("msg: "); break;
		case lt_warning : printf("warning: "); break;
		case lt_error 	: printf("error: "); break;
		case lt_fatal 	: printf("fatal: "); break;
	}

  va_list ap;
  va_start(ap, format);
	vprintf(format,ap);
  va_end(ap);

	printf("\n");

	if (stomped_int_fuckedup && *stomped_int_fuckedup != stomped_int_correct) {
		printf("The int was stomped on, it should be 0x%x and it is 0x%x\n",stomped_int_correct,*stomped_int_fuckedup);
	}

	if (do_assertions() && logtype == lt_assert) {
		//This is where I dump core so people can do a backtrace.
#ifndef KILL_ON_ASSERTION
		assert(0);
#else
		kill(getpid(),SIGTRAP);
#endif
	}
}


#ifdef _WIN32
LARGE_INTEGER time_freq()
{
	LARGE_INTEGER liTimerFrequency;
	QueryPerformanceFrequency( &liTimerFrequency );
	return liTimerFrequency;
}
#endif

//gets time of day in ms
unsigned int time_ms()
{
#ifdef _WIN32
	static LARGE_INTEGER liTimerFrequency = time_freq();
	LARGE_INTEGER liLastTime;
	QueryPerformanceCounter(&liLastTime);
	return ((double)liLastTime.QuadPart / liTimerFrequency.QuadPart)*1000;
#else
	struct timeval tv;
	struct timezone tz;
	tz.tz_minuteswest = 0;
	tz.tz_dsttime = 0;
	gettimeofday(&tv,&tz);
 	return (tv.tv_sec*1000)+(tv.tv_usec/1000);
#endif
}

//calcs the elapsed time between two time_ms()'es
unsigned int time_elapsed(unsigned int t_old, unsigned int t_new)
{
	if (t_old > t_new) //we crossed midnight
		return (24*60*60*1000+t_new)-t_old;
	else
		return t_new-t_old;
}

//-----------------------------------------------------------------------------
inline bool iswhitespace(char ch)
{
	return (ch == '\t') || (ch == ' ') || (ch == '\n') || (ch == '\r');
}

//-----------------------------------------------------------------------------
char *strtrim(char *string)
{
	//start by removing trailing whitespace:
	int last_char = strlen(string)-1;
	while (last_char >= 0 && iswhitespace(string[last_char])) {
		string[last_char] = 0;
		last_char--;
	}

	//then remove leading whitespace:
	while (*string && iswhitespace(*string)) {
		string++;
 	}
	return string;
}

//-----------------------------------------------------------------------------
bool expand_path(const char *source, char *target, size_t target_size)
{
	if (!*source) return false;

#ifndef _WIN32
	//see if there is a ~ to expand:
	if (source[0] == '~') {

	  char *static_source = NULL;
		struct passwd *passwd_entry=NULL;
		source++;
		
		if (*source == '/') {
			//current user:
    	static_source = (char*)source+1;
			passwd_entry = getpwuid(getuid());
		
		} else {
			//named user:
			char *slash = strchr(source, '/');
			if (slash) {
				char user_name[40];
				unsigned int user_name_length = slash-source;
				if (user_name_length > sizeof(user_name)-1)
					user_name_length = sizeof(user_name)-1;
        //Eero: we don't need to deal with users with longer names than 40 chars

	    	static_source = slash+1;
        strncpy(user_name,source,user_name_length);
				user_name[user_name_length] = 0;
				passwd_entry = getpwnam(user_name);
			}
		} 	

		if (!passwd_entry) {
			logmsg(lt_error,"Unable to get passwd entry for expand_path().");
			return false;
		}
	
		//snprintf() is GNU, replace with sprintf() on lesser platforms
		return snprintf(target,target_size,"%s/%s",passwd_entry->pw_dir,static_source) > 0;
	}
#endif

	//no ~ to expand, so just go about the boring buisness of copying the source to the target:
	if (strlen(source) < target_size)
		strcpy(target,source);

	return true;
}
