/***************************************************************************
                          fluff.h  -  description
                             -------------------
    begin                : Sat Oct 9 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk

    ~ expansion          : Sun Jan 30 2000
    copyright            : (C) 2000 by Eero Tamminen
    email                : t150315@cc.tut.fi
 ***************************************************************************/

/*
	This file defines a lot of unreleated stuff that is commonly used,
  but had no other logical place to be.
*/

#ifndef FLUFF_H
#define FLUFF_H

#include "systemheaders.h"


//The maximum number of objects on the server that is synced down to the clients
#define MAX_REAL_OBJECTS (2 << 16)

//The maximum number of objects that only exist on the client
#define MAX_ANON_OBJECTS (2 << 16)
//#define MAX_PLAYERS 128

#define IP_PORT 0xBABE

//physics defaults:
#define DEFAULT_ACCEL 40.0
#define DEFAULT_DRAG 0.01
#define KNOCK_BACK 0.001
#define RADIUS_DAMAGE 2.0
#define MAX_HEALTH 1000

//The number of weapons carried
#define WEAPON_SLOTS 5
//--------------------------------------------------------------------------

//types of objects that lives in the world:
typedef enum Object_type {
//player:
	ot_player_avatar=0,	
	ot_player_data,			
	ot_gib,							

//weapon invisibles:
	ot_weapon_test,
	ot_weapon_minigun,
	ot_weapon_shotgun,
	ot_weapon_megashotgun,
	ot_weapon_railgun,
	ot_weapon_plasmagun,
	ot_weapon_larpa,
	ot_weapon_rocket,
	ot_weapon_mininuke,
	ot_weapon_bignuke,
	ot_weapon_clusterbomb,
	ot_weapon_flamethrower,

//Other objects
 	ot_testobject, 			
	ot_explosion,     	
	ot_projectile,
	ot_fireball,			
	ot_hook,
	ot_scoreboard,			
	ot_railslug,
	ot_plasmaball,
	ot_tinygranade,
	ot_bignuke,
	ot_rocket,
	ot_clusterbomb,
	ot_grenade,

//Anon objects:
	ot_particle_blood,	
	ot_particle_debris,	
	ot_particle_railtrail,
	ot_particle_spark,
	ot_particle_smoke,

	//ot_error MUST be the last object type, as it is also the number of objecttypes
	//used for a flag when returnined from a function that didn't
	ot_error						
};

const char OBJECT_NAMES[ot_error+1][40] =
{
	"Player Avatar",
	"Player Data",
	"Giblet",

	"Experimental gun",
	"Minigun",
	"Shotgun",
	"Mega Shotgun",
	"Rail gun",
	"Plasma gun",
	"Larpa gun",
	"Rocket Launcher",
	"Mini nukegun",
	"Big nukegun",
	"Cluster bomb launcher",
	"Flame Thrower",

	"Test object",
	"Explosion",
	"Projectile",
	"Hook",
	"Score board",
	"Rail slug",
	"Plasma ball",
	"Fire ball",
	"Tiny granade",
	"Big nuke",
	"Rocket",
	"Clusterbomb",
	"Grenade",

	"Particle (blood)",
	"Particle (debris)",
	"Particle (railtrail)",
	"Particle (spark)",
	"Particle (smoke)",

	"ERROR"
};


//--------------------------------------------------------------------------
//gives a random integer in the range min..max
int rnd(int min, int max);

//makes a random color in the colorspace defined by c1 and c2
unsigned int mkrndcolor(unsigned int c1, unsigned int c2);

typedef enum Logtype {
	lt_fatal   = 0,
	lt_error   = 1,
	lt_warning = 2,
	lt_message = 3,
	lt_debug   = 4,
	lt_assert  = 5
};

extern Logtype chattiness;

void setmsgsource(char *source);
void setchattiness(Logtype new_chattiness);
inline bool do_assertions()
{
	return (chattiness >= lt_assert);
}

//error reporting / debugging.
void logmsg(Logtype logtype, char *format, ...);


unsigned int time_ms();
unsigned int time_elapsed(unsigned int t_old, unsigned int t_new);

bool iswhitespace(char ch);
char *strtrim(char *string);

//expands ~ in a path
bool expand_path(const char *source, char *target, size_t target_size);

//I use these to detect memory stomps:
extern int *stomped_int_fuckedup;
extern int stomped_int_correct;

//True when the user wishes to quit the program
extern bool request_quit;

#endif
