/***************************************************************************
                          font.cpp  -  description
                             -------------------
    begin                : Sun Jan 30 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "font_instance.h"
#include "fluff.h"
#include "loader.h"
#include <map>

class Font_glyphs : public std::map<int, Font_glyph*>{};

Font_instance::Font_instance()
{
	font_glyphs = new Font_glyphs;
}

//--------------------------------------------------------------------------------------
Font_instance::~Font_instance()
{
	Font_glyphs::iterator i= font_glyphs->begin();
	while (i!=font_glyphs->end()) {
  	delete i->second;
		i++;
	}	
	delete font_glyphs;
}

//--------------------------------------------------------------------------------------
bool Font_instance::load(Loader *loader, char *font_name, int32 color)
{
	char font_root[100];
	sprintf(font_root,"fonts/%s/",font_name);

	char spec_name[100];
	strcpy(spec_name,font_root);
	strcat(spec_name,"font.spec");
	int spec_size = loader->filesize(spec_name);
	
	if (spec_size <= 0) {
  	logmsg(lt_error,"Unable to load the font spec: %s",spec_name);
		return false;
	}

	char *font_spec = (char*)malloc(spec_size+1);
	font_spec[spec_size] = 0;
	if (!loader->fileread(spec_name,font_spec,spec_size,0)) {
		free(font_spec);
		logmsg(lt_error,"Unable to load the fontspec: %s",spec_name);
		return false;
	}

	space_advance = 10;	
	char *spec_line = strsep(&font_spec,"\n");
	while (spec_line && *spec_line) {
		Font_glyph *glyph = new Font_glyph;
		
		if (glyph->load_glyph(&loader->graphics,font_root,spec_line,color)) {
    	font_glyphs->insert(std::pair<int, Font_glyph *>(glyph->get_char(),glyph));
			if ((char)glyph->get_char() == 'M') space_advance = glyph->get_advance();
		} else {
      delete glyph;
			logmsg(lt_error,"Unable to load glyph for spec: %s",spec_line);
		};

		spec_line = strsep(&font_spec,"\n");
	}

	return true;
}

//--------------------------------------------------------------------------------------
int Font_instance::get_text_length(char *text)
{
	return 0;
}

//--------------------------------------------------------------------------------------
int Font_instance::get_text_height(char *text)
{
	return 0;
}

//--------------------------------------------------------------------------------------
bool Font_instance::put_text(int xpos, int ypos, char *text, Mutable_raw_surface *target)
{
	int text_length = strlen(text);
	for(int c=0;c<text_length;c++) {
  	Font_glyphs::iterator glyph = font_glyphs->find(text[c]);
		if (glyph != font_glyphs->end()) {
	  	glyph->second->put_glyph(target,xpos,ypos);	
		} else {
			if (text[c] == ' ')
				xpos += space_advance;
			else
				logmsg(lt_error,"Unable to find a glyph for character '%c'",text[c]);
		}
	}

	return true;
}
