/***************************************************************************
                          fonts.cpp  -  description
                             -------------------
    begin                : Mon Feb 7 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "fonts.h"
#include "font_instance.h"
#include "loader.h"
#include "fluff.h"
#ifdef HAS_STL_HASHMAP
#include <hash_map>
#else
#include <map>
#endif

#ifdef HAS_STL_HASHMAP

struct Streq_ {
	bool operator()(const char* s1, const char* s2) const
  {
		return strcmp(s1, s2) == 0;
	}
};

class Font_instances : public std::hash_map<const char *,Font_instance *,hash<const char*>,Streq_>{};

#else

struct Strless_ {
	bool operator()(const char* s1, const char* s2) const
  {
		return strcmp(s1, s2) < 0;
	}
};

class Font_instances : public std::map<const char *,Font_instance *,Strless_>{};

#endif

//----------------------------------------------------------------------------
Fonts::Fonts()
{
	font_instances = new Font_instances;
	loader = NULL;
}

//----------------------------------------------------------------------------
void Fonts::init(Loader *loader_)
{
	loader = loader_;
}

//----------------------------------------------------------------------------
Fonts::~Fonts()
{
	Font_instances::iterator i,e,t;
	i=font_instances->begin();
	e=font_instances->end();
	while (i!=e) {
		t=i;
		++i;
		font_instances->erase(t);
  	delete t->first;
  	delete t->second;
	}
	delete font_instances;
}

//----------------------------------------------------------------------------
Font_instance *Fonts::getfont(const char *name, int size, int32 color, bool antialize)
{
	/*
  	NOTE: The ttf2rgbf project should be merged into this class so that
		prerendering fonts is not needed.
	*/

	char font_name[80];
	sprintf(font_name,"%s_%i",name,size);

	Font_instances::iterator i=font_instances->find(font_name);
	if (i!=font_instances->end())
  	return i->second;
	
	Font_instance *newfont = new Font_instance;
	if (!newfont->load(loader,font_name,color)) {
		delete newfont;
		logmsg(lt_error,"Unable to load the font %s, wonkyness ahead",font_name);
		return NULL;
	}	

	char *namecopy = new char[strlen(font_name)+1];
	strcpy(namecopy,font_name);
	font_instances->insert(std::pair<const char *,Font_instance *>(namecopy,newfont));
	return newfont;
}

