/***************************************************************************
                          graphic.h  -  description
                             -------------------
    begin                : Sun Oct 17 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#ifndef GRAPHIC_H
#define GRAPHIC_H

//#include "systemheaders.h"
#include "raw_surface.h"

/**Class to hold a single image
  *@author Flemming Frandsen
  */
class Loader;
class Graphic {
public:
	Graphic();
	~Graphic();

	void create(int xsize,int ysize) {
		if (surface && !surface->is_mutable()) {
			delete surface;
			surface=0;
		}
		if (!surface) {
			surface=new Mutable_raw_surface();
		}
		dynamic_cast<Mutable_raw_surface *>(surface)->create(xsize,ysize);
	}

	bool is_mutable() const {
		return surface && surface->is_mutable();
	}

	void create(Graphic *graphic) {
		if (surface && !surface->is_mutable()) {
			delete surface;
			surface=0;
		}
		if (!surface) {
			surface=new Mutable_raw_surface();
		}
		dynamic_cast<Mutable_raw_surface *>(surface)->create(graphic->get_surface());
	}

	bool load(Loader *loader, char *name);

	void maketransparent();
	inline const Raw_surface *get_surface() const { return surface; }
	inline Mutable_raw_surface *get_mutable_surface() const {
		if (get_surface() && get_surface()->is_mutable()) {
			return (Mutable_raw_surface *)get_surface();
		} else {
			return 0;
		}
	}

	int32 *get_pixels_write() const {
		return get_surface()?get_surface()->get_pixels_write():0;
	}

	void apply_color_under_offset( int32 color, int base ) {
		if (!get_mutable_surface()) { return; }
		get_mutable_surface()->apply_color_under_offset(color,base);
	}

	const int32 *get_pixels_read() const {
		return get_surface()?get_surface()->get_pixels_read():0;
	}

	/* proxies for Raw_surface */
	int get_xsize() const { return get_surface()?get_surface()->get_xsize():-1; }
	int get_ysize() const { return get_surface()?get_surface()->get_ysize():-1; }

	void draw_copy_stretch(Mutable_raw_surface *target,
												 int source_xpos,  int source_ypos,
												 int source_xsize, int source_ysize,
												 int target_xpos,  int target_ypos,
												 int enlargement = 1) const {
		if (!get_surface()) { return; }
		get_surface()->draw_copy_stretch(target,
																		 source_xpos,source_ypos,
																		 source_xsize,source_ysize,
																		 target_xpos,target_ypos,
																		 enlargement);
	}

/*  	void draw_copy(Mutable_raw_surface *target, */
/*  								 int source_xpos, int source_ypos, */
/*  								 int target_xpos, int target_ypos, */
/*  								 int target_xsize, int target_ysize) const { */
/*  		if (!get_surface()) { return; } */
/*  		get_surface()->draw_copy(target, */
/*  														 source_xpos,source_ypos, */
/*  														 target_xpos,target_ypos, */
/*  														 target_xsize,target_ysize); */
/*  	} */

	void draw_alpha(Mutable_raw_surface *target, int xpos, int ypos) const {
		if (!get_surface()) { return; }
		get_surface()->draw_alpha(target,xpos,ypos);
	}

	void draw_view(Mutable_raw_surface *target,
												 int xpos, int ypos) const {
		if (!get_surface()) { return; }
		get_surface()->draw_view(target,xpos,ypos);
	}

	void draw_map(Mutable_raw_surface *target_map,
								Mutable_raw_surface *target_bgmap,
								int xpos, int ypos) const {
		if (!get_surface()) { return; }
		get_surface()->draw_map(target_map,target_bgmap,xpos,ypos);
	}

protected:
	Raw_surface *surface;
};

#endif
