/***************************************************************************
                          keyboard.cpp  -  description
                             -------------------
    begin                : Thu Oct 28 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "keyboard.h"
#include "fluff.h"
#include "systemheaders.h"

Keyboard::Keyboard(Display *display_)
{
	display = display_;
	XAutoRepeatOff(display);
}


Keyboard::~Keyboard()
{
	XAutoRepeatOn(display);
	//destroy things
}

void Keyboard::clear()
{
	buffer_inuse = 0;
	keysdown_inuse = 0;
}

Keyboardkey Keyboard::getkey()
{
	poll();

	Keyboardkey key;
	key.sym = 0; key.ascii = 0;
	if (buffer_inuse) {
  	key = buffer[0];
		buffer_inuse--;
		for (int c=0; c<buffer_inuse;c++)
			buffer[c] = buffer[c+1];
	}
	return key;
}

bool Keyboard::ispressed(int keysym)
{
	poll();
	for (int c=0;c<keysdown_inuse;c++)
		if (keysdown[c] == keysym)
			return true;
	return false;
}

void Keyboard::poll()
{
	XEvent e;
	while (XCheckMaskEvent(display, KeyPressMask | KeyReleaseMask, &e)) {
		Keyboardkey newkey;
	
		newkey.sym = XLookupKeysym(&e.xkey,0);
		newkey.ascii = 0;

		//find the ascii char:
		char buf[11];
		buf[10]=0;
		XLookupString(&e.xkey, buf, 10, NULL, NULL);
		if (strlen(buf) == 1) newkey.ascii = buf[0];

		//add the key to the input buffer
		if (e.type != KeyRelease)
			buffer[buffer_inuse++] = newkey;

		if (e.type != KeyRelease) {
			//Add the key at the end of the pressed keys
			if (!ispressed(newkey.sym))
				keysdown[keysdown_inuse++] = newkey.sym;

		} else {

			//remove the key from the set if it can be found
			bool found = false;
			for (int c=0;c<keysdown_inuse;c++) {
				if (keysdown[c] == newkey.sym) found = true;
				if (found) keysdown[c] = keysdown[c+1];
			}
			if (found) keysdown_inuse--;
		}
	}
}


char *Keyboard::getkeyname(Keyboardkey key)
{
	//default to the ascii char
	keynamebuffer[0] = key.ascii;
	keynamebuffer[1] = 0;

	//handle the common keys:
	switch (key.sym) {
		case XK_Shift_L			: strcpy(keynamebuffer,"Left shift"); break;
		case XK_Shift_R		  : strcpy(keynamebuffer,"Right shift"); break;
		case XK_Control_L   : strcpy(keynamebuffer,"Left ctrl"); break;
		case XK_Control_R	  : strcpy(keynamebuffer,"Right ctrl"); break;
		case XK_Caps_Lock	  : strcpy(keynamebuffer,"Caps lock"); break;
		case XK_Alt_L	      : strcpy(keynamebuffer,"Left alt"); break;
		case XK_Mode_switch:
		case XK_Alt_R	      : strcpy(keynamebuffer,"Alt gr"); break;
		case XK_space       : strcpy(keynamebuffer,"Space"); break;
		case XK_BackSpace   : strcpy(keynamebuffer,"Backspace"); break;
		case XK_Tab			    : strcpy(keynamebuffer,"Tab"); break;
		case XK_Linefeed	  : strcpy(keynamebuffer,"Linefeed"); break;
		case XK_Clear		    : strcpy(keynamebuffer,"Clear"); break;
		case XK_Return		  : strcpy(keynamebuffer,"Return"); break;
		case XK_Pause		    : strcpy(keynamebuffer,"Pause"); break;
		case XK_Scroll_Lock : strcpy(keynamebuffer,"Scroll lock"); break;
		case XK_Sys_Req     : strcpy(keynamebuffer,"Sys req"); break;
		case XK_Escape	    : strcpy(keynamebuffer,"Esc"); break;
		case XK_Delete	    : strcpy(keynamebuffer,"Delete"); break;
		case XK_Home			  : strcpy(keynamebuffer,"Home"); break;
		case XK_Left			  : strcpy(keynamebuffer,"Left"); break;
		case XK_Up			    : strcpy(keynamebuffer,"Up"); break;
		case XK_Right		    : strcpy(keynamebuffer,"Right"); break;
		case XK_Down			  : strcpy(keynamebuffer,"Down"); break;
		case XK_Page_Up	    : strcpy(keynamebuffer,"Page Up"); break;
		case XK_Page_Down	  : strcpy(keynamebuffer,"Page Down"); break;
		case XK_End			    : strcpy(keynamebuffer,"End"); break;
		case XK_Begin		    : strcpy(keynamebuffer,"Begin"); break;
		case XK_Insert			: strcpy(keynamebuffer,"Insert"); break;
	}

	//doh, gotta be user-hostile now:
	if (!keynamebuffer[0]) sprintf(keynamebuffer,"<%x>",key.sym);

	return keynamebuffer;
}

