/***************************************************************************
                          nil_client.cpp  -  description
                             -------------------
    begin                : Mon Nov 15 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#include "systemheaders.h"
#include "configuration.h"
#include "nil_client.h"
#include "fluff.h"
#include "loader.h"
#include "all.h"
#include "controls.h"
#include "loader.h"
#include "graphic_bar.h"
#include "icon_graph.h"
#include "font_instance.h"
#include "game_client.h"
#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"
#include "soundsystem.h"

void run_client(Configuration &config)
{
	setmsgsource("C");
	logmsg(lt_message,"Running client");

	int sdl_flags = /*SDL_INIT_AUDIO |*/ SDL_INIT_VIDEO;

	if (config.get_option_int("coredump"))
		sdl_flags |= SDL_INIT_NOPARACHUTE;
	int sdl_error = SDL_Init(sdl_flags);
	if (sdl_error < 0) {
		logmsg(lt_fatal,"Unable to initialize SDL: %i",sdl_error);
		return;
	}

	atexit(SDL_Quit);

	Config_xy screen_size = config.get_option_xy("screen_size");
	int sdl_video_flags = SDL_HWSURFACE;
	if (config.get_option_int("fullscreen"))
		sdl_video_flags |= SDL_FULLSCREEN;
	SDL_Surface *screen = SDL_SetVideoMode(screen_size.x, screen_size.y, 32, sdl_video_flags);
	if (!screen) {
		logmsg(lt_fatal,"Couldn't set %ix%ix32 video mode: %s\n",screen_size.x, screen_size.y,SDL_GetError());
		return;
	}
	logmsg(lt_debug,"SDL initialized.\n");

	logmsg(lt_debug,"Going to add: %s as sources",config.get_option("data"));
	Loader loader;
	loader.addsources(config.get_option("data"));

	Mix_OpenAudio(22100, AUDIO_U8, 1, 1024);
	int channels = Mix_AllocateChannels(48);
	logmsg(lt_debug,"Mixer initialized, %i channels.",channels);

	Soundsystem soundsystem(&loader);

/*
	This seems to work quite nicely  -lSDL_mixer is needed for this

	Mix_OpenAudio(22100, AUDIO_U8, 1, 1024);

	int channels = Mix_AllocateChannels(48);
	logmsg(lt_debug,"mixer channels: %i",channels);

	void *sample_ptr = loader.getfile("sound/9mm.wav");
	int sample_size = loader.filesize("sound/9mm.wav");
	logmsg(lt_debug,"size of test sample: %i",sample_size);

	Mix_Chunk *chunk = Mix_LoadWAV_RW(SDL_RWFromMem(sample_ptr, sample_size), 1);

	int i = 0;
	while (1) {
		Mix_PlayChannel(-1,chunk,0);
		i = (i+1) % 8;
		logmsg(lt_debug,"%i",i);
	  usleep(100000);
	}
return;
*/

	Keymapper keymapper;
	keymapper.init(&config);

	//initialize the single player client:
  Game_client game_client;
	game_client.init(&loader, &soundsystem, config.get_option("player_name"), config.get_option_int("player_color"));

	int view_quality = config.get_option_int("view_quality");
	game_client.setgeom(0,0,screen_size.x/view_quality,screen_size.y/view_quality,view_quality);

	int join_error=game_client.join_game(config.get_option("server"),config.get_option_int("server_port"));

	if (join_error) {
		logmsg(lt_fatal,"Unable to join the game (%i), giving up and going home",join_error);
	}

	//for stats:
	Font_instance *smallfont = loader.fonts.getfont("verdana",9);

	bool done = false;
	Obj_player_data *player_data = NULL;

	Icon_graph traffic_graph;
	traffic_graph.init(10,20,50,20,PIXELA(128,128,0,6),PIXELA(255,0,0,8));

	float traffic_trend = 0;
	float last_traffic_sample = 0;
	float traffic_sum = 0;
	bool plot_traffic = false;

	Graphic *logo=loader.graphics.getgraphic("logos/nil_small.rgbf");

	bool screen_shoot = false;
	bool show_logo = true;
	int screen_shot_number = 0;
	bool death_cam_armed = false;
	bool death_cam_active = false;
	int anon_anounce=0;

  while (!done) {

		if (request_quit)
			done = true;

		if (anon_anounce-- <= 0) {
			anon_anounce = 30;
			logmsg(lt_debug,"Anon objects: %i, time:%.2f",game_client.clientworld.get_anon_objects_inuse(),game_client.clientworld.gettime());
		}

		//handle the SDL events:
		SDL_PumpEvents();
		SDL_Event event;
		while (SDL_PollEvent(&event)) {

			//handle the keyboard events:
			if (event.type==SDL_KEYUP || event.type==SDL_KEYDOWN) {
				Nil_keyevent keyevent;
				keyevent.sdl_keyevent = (SDL_KeyboardEvent*)&event;
				keymapper.lookup(keyevent);
			
				int target = keyevent.keyfunction & KEYFUNCTION::TARGET_MASK;

				switch (target) {
					case (KEYFUNCTION::PLAYER1): {
						game_client.handle_key(&keyevent);
						break;
					}

					case (KEYFUNCTION::PLAYER2): {
						//player_controls.handle_key((SDL_KeyboardEvent*)&event);
						break;
					}

					case (KEYFUNCTION::HOTKEY): {
						if (keyevent.sdl_keyevent->state != SDL_PRESSED) break;

						//handle gen purpose hotkeys here...
						int function = keyevent.keyfunction & KEYFUNCTION::FUNCTION_MASK;
						switch (function) {

  						case (KEYFUNCTION::MENU) :{
              	done = true;
								break;
							}		
		
							case KEYFUNCTION::SCREEN_SHOOT :{
              	screen_shoot = true;
								break;
							}

							case KEYFUNCTION::TOGGLE_TRAFFIC : {
								plot_traffic = !plot_traffic;
								if (plot_traffic)
									logmsg(lt_debug,"plotting of traffic on (100%% is 128kb/s)");
								else
									logmsg(lt_debug,"plotting of traffic off");
								break;
							}

							case KEYFUNCTION::TOGGLE_SHADOW : {
								game_client.dropshadow = !game_client.dropshadow;
								if (game_client.dropshadow)
									logmsg(lt_message,"Dropshadow has been turned on");
								else
									logmsg(lt_message,"Dropshadow has been turned off");
								break;
							}

							case KEYFUNCTION::TOGGLE_SMOOTH : {
								game_client.smooth = !game_client.smooth;
								if (game_client.smooth)
									logmsg(lt_message,"Smoothing has been turned on");
								else
									logmsg(lt_message,"Smoothing has been turned off");
								break;
							}

							case KEYFUNCTION::TOGGLE_LOGO : {
								show_logo = !show_logo;
								if (show_logo)
									logmsg(lt_debug,"The nifty logo is back.");
								else
									logmsg(lt_debug,"The nifty logo now gone, why oh why?");

								break;
							}

							case KEYFUNCTION::TOGGLE_DEATHCAM : {
								death_cam_active = !death_cam_active;
								if (death_cam_active)
									logmsg(lt_debug,"The DeathCam(tm) is now active.");
								else
									logmsg(lt_debug,"The DeathCam(tm) is now off.");

								break;
							}
							
						} //switch (function) {
						break;
					} //case (KEYFUNCTION::HOTKEY): {
			
					default: {
						logmsg(lt_warning,"Key with invalid target: %x", keyevent.keyfunction);
					}   	
				} //switch (target) {
	

				if (event.type==SDL_KEYDOWN) {
					//a couple of keys for debugging:
					switch (keyevent.sdl_keyevent->keysym.sym) {
						case SDLK_q: game_client.clientworld.dump_objects(); break;
    	    	case SDLK_w: game_client.clientworld.getscoreboard()->dump(); break;
						default:{}
					}
				}
		
			} //if (event.type==SDL_KEYUP || event.type==SDL_KEYDOWN) {

		} //while (SDL_PollEvent(&event)) {

    if (!player_data)	{
			player_data = (Obj_player_data *)game_client.clientworld.get_object(game_client.clientworld.getowner(),ot_player_data);
			if (!player_data) logmsg(lt_warning,"Couldn't find player_data");	
		}
		game_client.think();

		Mutable_raw_surface tmp;
		if (SDL_MUSTLOCK(screen)) {
			if ( SDL_LockSurface(screen) < 0 )
				logmsg(lt_error,"Unable to lock SDL surface");		
		}
		tmp.attach(screen_size.x,screen_size.y,(int32 *)screen->pixels);

		game_client.draw(&tmp);

		if (show_logo) {
			logo->draw_alpha(&tmp,tmp.get_xsize()-logo->get_xsize()-2,15);
		}

		//write fps:
		{
			char str[140];
			float tick_time = game_client.get_trend_tick_duration();
			if (tick_time == 0) tick_time = 1;
			float fps = 1/tick_time;
			sprintf(str,"fps: %.2f, %d %d",fps,game_client.clientworld.getobjcount(), game_client.clientworld.get_anon_objects_inuse());
			if (smallfont)
				smallfont->put_text(10,20,str,&tmp);
		}

		if (plot_traffic) {
			traffic_sum += game_client.last_update_size;
			float traffic_time = game_client.clientworld.gettime()- last_traffic_sample;
			if (traffic_time > 1) {
				traffic_trend += ((traffic_sum/traffic_time)-traffic_trend)/5;

				traffic_sum = 0;
				last_traffic_sample = game_client.clientworld.gettime();
	  	  traffic_graph.addsample(traffic_trend/(128*1024/8));
			}
			traffic_graph.draw(&tmp);
		}

		if (death_cam_active) {
			if (game_client.clientworld.time_of_death >= 0 && (game_client.clientworld.gettime() - game_client.clientworld.time_of_death) == 0)
				death_cam_armed = true;
	
			if (death_cam_armed && (game_client.clientworld.gettime() - game_client.clientworld.time_of_death) > 0.2) {
				screen_shoot = true;
				death_cam_armed = false;
			}
		} else
			death_cam_armed = false;
	
		if (screen_shoot) {
			screen_shoot = false;
			screen_shot_number++;
	
			char filename[100];
			sprintf(filename,"/tmp/nil_screenshot_%i.rgbf",screen_shot_number);
			logmsg(lt_message,"Dumping screenshot to: %s",filename);

			//Make sure that all the pixels are visible:
			int32 *pixels = tmp.get_pixels_write();
			for (int y=0;y<tmp.get_ysize();y++)
				for (int x=0;x<tmp.get_xsize();x++) {
					int32 *pixel = pixels + x+y*tmp.get_xsize();
					*pixel |= 0xff000000;
				}
			tmp.save_to_file(filename);
		}

		if ( SDL_MUSTLOCK(screen) ) {
			SDL_UnlockSurface(screen);
		}

		SDL_UpdateRect(screen, 0,0, screen_size.x, screen_size.y);
  }
	return;
}
