/***************************************************************************
                          nil_files.h  -  description
                             -------------------
    begin                : Sun Feb 6 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk

 This file defines the different structures used in files around NiL

Note: always call swap on the structs before writing and after reading them.

 ***************************************************************************/


#ifndef NIL_FILES_H
#define NIL_FILES_H

#ifdef _MSC_VER
#pragma warning (disable : 4091)
#endif

//-----------------------------------------------------------------------------
typedef int Int32;
typedef short int Int16;

void swap32(Int32 &value);
void swap16(Int16 &value);

//-----------------------------------------------------------------------------
const Int32 NIL_MAGIC = 0xbabe0000;

/*
	Lump files are constructed like this:

Lump_header
data for file 1
data for file 2
data for file 3

Lump_directory_header

Lump_directory_item for file 1
file name of file 1

Lump_directory_item for file 2
file name of file 2

Lump_directory_item for file 3
file name of file 3

*/

//-----------------------------------------------------------------------------
const Int32 LUMP_HEADER_MAGIC = NIL_MAGIC+1;
typedef struct Lump_header {
	Int32 magic;
	Int32 directory_offset;

	void swap() {
  	swap32(magic);
  	swap32(directory_offset);
	}
};

//-----------------------------------------------------------------------------
const Int32 LUMP_DIRECTORY_HEADER_MAGIC = NIL_MAGIC+2;
typedef struct Lump_directory_header {
	Int32 magic;
	Int32 items;
	
	void swap() {
  	swap32(magic);
  	swap32(items);
	}
};

//-----------------------------------------------------------------------------
const Int32 LUMP_DIRECTORY_ITEM_MAGIC = NIL_MAGIC+3;
typedef struct Lump_directory_item {
	Int32 magic;
	Int32 file_offset;			
	Int32 file_length;			
	Int32 name_length;			

	void swap() {
  	swap32(magic);
  	swap32(file_offset);
  	swap32(file_length);
  	swap32(name_length);
	}
};

//-----------------------------------------------------------------------------
const Int32 RGBF_HEADER_MAGIC = NIL_MAGIC+4;
typedef struct Rgbf_header {
	Int32 magic;
	Int32 xsize;			
	Int32 ysize;			

	void swap() {
  	swap32(magic);
  	swap32(xsize);
  	swap32(ysize);
	}
};


#endif