/***************************************************************************
                          object.cpp  -  description
                             -------------------
    begin                : Fri Oct 8 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#include "obj_base.h"
#include "fluff.h"
#include "world.h"
#include "all.h"
#include "nil_math.h"
#include "systemheaders.h"

void Obj_base::newton_update(float newtime)
{
	if (curtime == -1) {
		curtime	= newtime; //This shouldn't happen, if it does then it means that the object was not initialized
		createtime = newtime;
		return;
	}

	if (world->isclient()) {
		//save the old physical state for later
		old_xpos=xpos;
		old_ypos=ypos;
		old_xspeed=xspeed;
		old_yspeed=yspeed;
	}
	
	float dt = newtime-curtime;

	//speed
	xspeed = xspeed   -SQR(xspeed)*SIGN(xspeed)*air_resistance*dt;
	yspeed = yspeed +(-SQR(yspeed)*SIGN(yspeed)*air_resistance + yaccel)*dt;

	//position
	xpos = xpos+xspeed*dt;
	ypos = ypos+yspeed*dt;

	//time
	curtime = newtime;
}

Map_hit Obj_base::update_simple(bool check_object_collision)
{
	Map_hit map_hit;
	map_hit.hittype = ht_none; //if nothing happens we didn't hit anything

	//save the old physical state for later
	old_xpos=xpos;
	old_ypos=ypos;
	old_xspeed=xspeed;
	old_yspeed=yspeed;
	
	client_think();

	//check to see if we have hit a pixel
	int xc=0;
	int yc=0;
	bool collided = false;
	if (radius > 0)
		collided = world->collision(ROUND(old_xpos),ROUND(old_ypos), ROUND(xpos), ROUND(ypos), xc, yc, radius);
	else
		collided = world->collision(ROUND(old_xpos),ROUND(old_ypos), ROUND(xpos), ROUND(ypos), xc, yc, this);

 	if (collided) {
		map_hit.hittype = ht_map; //we hit a pixel on the map
		map_hit.xc = xc;
		map_hit.yc = yc;	
		xpos = xc;
		ypos = yc;
	};

	if (ROUND(xpos) < 0) {
		map_hit.hittype = ht_left;

		ypos = old_ypos;//fixme: calc edge intersection in stead
		xpos = 0;

		map_hit.xc = ROUND(xpos);
		map_hit.yc = ROUND(ypos);	
	}

	if (ROUND(ypos) < 0) {
		map_hit.hittype = ht_top;

		ypos = 0;
		xpos = old_xpos; //fixme: calc edge intersection in stead

		map_hit.xc = ROUND(xpos);
		map_hit.yc = ROUND(ypos);	
	}

	if (ROUND(xpos) >= world->map_buffer.get_xsize()) {
		map_hit.hittype = ht_right;

		ypos = old_ypos; //fixme: calc edge intersection in stead
		xpos = world->map_buffer.get_xsize()-1;

		map_hit.xc = ROUND(xpos);
		map_hit.yc = ROUND(ypos);	
	}

	if (ROUND(ypos) >= world->map_buffer.get_ysize()) {
		map_hit.hittype = ht_bottom;

		xpos = old_xpos; //fixme: calc edge intersection in stead
		ypos = world->map_buffer.get_ysize()-1;

		map_hit.xc = ROUND(xpos);
		map_hit.yc = ROUND(ypos);	
	}

	//check to see if we have hit any hurtable objects
	if (check_object_collision) {
		
		if (((Serverworld*)world)->hit_object(
				old_xpos, old_ypos, xpos, ypos,
				map_hit.xc, map_hit.yc, map_hit.objectid)) {

			map_hit.hittype = ht_object;
			xpos = map_hit.xc;
			ypos = map_hit.yc;
		}
	}

	//sanity check:
	if (ROUND(xpos) >= world->map_buffer.get_xsize())
		xpos = world->map_buffer.get_xsize()-1;
		
	if (ROUND(ypos) >= world->map_buffer.get_ysize())
			ypos = world->map_buffer.get_ysize()-1;

	if (ROUND(ypos) < 0)
		ypos = 0;

	if (ROUND(xpos) < 0)
		xpos = 0;

	return map_hit;
}

Obj_base::Obj_base(World *world_, int id)
{
	world = world_;
	done = false;
	net_dirty = true;

	objectid = id;

	xpos = 0;
	ypos = 0;
	xspeed = 0;
	yspeed = 0;
	yaccel = DEFAULT_ACCEL;
	air_resistance = DEFAULT_DRAG;

	//the time this data was updated.
	curtime = -1;

	ownerid = -1;

	radius = 1;

	createtime = world->gettime();
}

Obj_base::~Obj_base()
{
//	logmsg(lt_debug,"deleted Obj_base");
}

void Obj_base::serialize(Serializer *serializer)
{
	serializer->rw(ownerid);
	serializer->rw(yaccel);
	serializer->rw(air_resistance);
	serializer->rw(createtime);
	serializer->rw(curtime);
	serializer->rw(xpos);
	serializer->rw(ypos);
	serializer->rw(xspeed);
	serializer->rw(yspeed);
}

void Obj_base::client_think()
{
	newton_update(world->gettime());
}

void Obj_base::bogo_bounce()
{
	//bounce if we are moving very fast, otherwise just sit still.
	if (ABS(xspeed) > 15 || ABS(yspeed) > 15) {
		if (ABS(xspeed) >  ABS(yspeed)) {
			xspeed *= -0.4;
			yspeed *= 0.4;
		} else {
			xspeed *= 0.4;
			yspeed *= -0.4;
		}
	} else {
		xspeed = 0;
		yspeed = 0;
	}
}

bool Obj_base::canmove()
{
	//return world->canmove(ROUND(xpos),ROUND(ypos));
	return true;
}

bool Obj_base::onground()
{
	return !IS_PASSABLE(world->map_buffer.get_pixels_read()[ROUND(xpos)+(ROUND(ypos)+1)*world->map_buffer.get_xsize()]);
}

Obj_player_data *Obj_base::get_owner_data()
{
	return (Obj_player_data *)world->get_object(ownerid,ot_player_data);
}

Obj_player_avatar *Obj_base::get_owner_avatar()
{
	Obj_player_data * od = get_owner_data();
	if (od)
		return od->get_player_avatar();
	else
		return NULL;
}





















