/***************************************************************************
                          obj_particle.cpp  -  description
                             -------------------
    begin                : Thu Nov 4 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "obj_particle.h"
#include "fluff.h"
#include "clientworld.h"

//-----------------------------------------------------------------------------
Obj_particle::Obj_particle(World *world_, int id) : Obj_base(world_,id)
{
	air_resistance = DEFAULT_DRAG*2;
}

//-----------------------------------------------------------------------------
Obj_particle::~Obj_particle()
{
}

//-----------------------------------------------------------------------------
void Obj_particle::init(float x, float y, float xspeed_, float yspeed_)
{
	xpos = x;
	ypos = y;
	xspeed = xspeed_;
	yspeed = yspeed_;
}

//-----------------------------------------------------------------------------
void Obj_particle::client_think()
{
	old_xpos=xpos;
	old_ypos=ypos;
	old_xspeed=xspeed;
	old_yspeed=yspeed;
	
	newton_update(world->gettime());

	//check to see if we have hit a pixel
	int xc=0;
	int yc=0;
	bool collided = false;
	collided = world->collision(ROUND(old_xpos),ROUND(old_ypos), ROUND(xpos), ROUND(ypos), xc, yc, 1);

 	if (collided) {
		Vector v;
		v.x = xspeed;
		v.y = yspeed;
		v = normalize(v);
		int x = ROUND(xc + v.x);
		int y = ROUND(yc + v.y);

		if (x >= 0 && y >= 0 && x < world->map_buffer.get_xsize() && y < world->map_buffer.get_ysize()) {
			xpos = xc;
			ypos = yc;
			hit_something(x+y*world->map_buffer.get_xsize(), xc+yc*world->map_buffer.get_xsize());
		} else
			done = true; //if we have hit some something outside the map, just die.
	};
//	if(age()>10) done=true;	
}

//-----------------------------------------------------------------------------
void Obj_particle::draw(Viewport *viewport)
{
	int x = ROUND(xpos)-viewport->xpos;
	int y = ROUND(ypos)-viewport->ypos;

	if (x >= 0 && y >= 0 && x < viewport->get_xsize() && y < viewport->get_ysize()) {
		view_draw(viewport->get_pixels_write()+x+y*viewport->get_xsize());
	}
}

//=============================================================================
//= Bloodloss in a bathroom stall, scartissue that I wish you saw...
//=============================================================================
void Obj_particle_blood::view_draw(int32 *target)
{
	*target = color | PF_TYPE_EARTH;
}

void Obj_particle_blood::hit_something(int offset_after_hit, int offset_before_hit)
{
	int32 *pixel = world->map_buffer.get_pixels_write()+offset_after_hit;

	//only stain non passable pixels.
	if (!IS_PASSABLE(*pixel))
		*pixel = color | (*pixel & (PF_ALPHA_BITS | PF_TYPE_BITS));

	done = true;
}

//=============================================================================
//= Debris
//=============================================================================
void Obj_particle_debris::view_draw(int32 *target)
{
	*target = color | PF_TYPE_EARTH;
}

void Obj_particle_debris::hit_something(int offset_after_hit, int offset_before_hit)
{
/*
	int32 *pixel = world->map_buffer.pixels+offset_after_hit;

	//only stain non passable pixels.
	if (!IS_PASSABLE(*pixel))
		*pixel = color | (*pixel & (PF_ALPHA_BITS | PF_TYPE_BITS));
*/

	bogo_bounce();	

	if (age() > 1)
		done = true;
}

//=============================================================================
//= Railtrail
//=============================================================================
void Obj_particle_railtrail::view_draw(int32 *target)
{
	int alpha = ROUND(15-(15*age()/2));
	if (alpha > 15) alpha = 15;
	if (alpha <= 0) {
  	done = true;
	} else {
		int32 aa = PIXELA(0,0,0,alpha);

		if (!IS_PASSABLE(*target)) return;

  	int32 type = *target & PF_TYPE_BITS;
	
   	if (IS_OPAQUE(aa))
			*target = (*target & PF_RGB_BITS) | type;

		else if (IS_INVISIBLE(aa)) {
			//Don't modify the pixel at all
   	
		}	else {
    	//ARGH! ARGH! it's not opaque and it's not invisible, so I gotta do alpha now.
 	  	
			int a = alpha;
			int a1 = 16-a;
  		
			int r = (GETRED(color)*a   + GETRED(*target)*a1)   >> 4;
			int g = (GETGREEN(color)*a + GETGREEN(*target)*a1) >> 4;
			int b = (GETBLUE(color)*a  + GETBLUE(*target)*a1)  >> 4;
  		
			*target = PIXEL(r,g,b) | type;
		}
	}
}

//-----------------------------------------------------------------------------
void Obj_particle_railtrail::hit_something(int offset_after_hit, int offset_before_hit)
{
	done = true;
}

//=============================================================================
//= Spark
//=============================================================================
//-----------------------------------------------------------------------------
void Obj_particle_spark::view_draw(int32 *target)
{
	const int32 *map_pixel = world->map_buffer.get_pixels_read()+ROUND(xpos)+ROUND(ypos)*world->map_buffer.get_xsize();
	if (!IS_PASSABLE(*map_pixel)) {
  	done = true;
    return;
	}

	float new_age = age();
	if (new_age != old_age) {
		temp /= 1+(new_age-old_age)/2;
		old_age = new_age;
		temp -= 1;
		if (temp < 0) temp = 0;
	}

  float red 	= temp;
	float green = red / 3;
	float blue 	= red / 10;
	if (red > 255) {
		green += (red-255)/2;
		blue  += (red-255)/2;
  	red = 255;
	}

	if (green > 255) {
		blue  += (green-255)/2;
  	green = 255;
	}

	if (blue > 255) {
  	blue = 255;
	}

  color = PIXELA(ROUND(red),ROUND(green),ROUND(blue),15);	
	*target = color;

	//Make bloody sure that the object dies.
	if (age() > 15) done = true;
}

//-----------------------------------------------------------------------------
void Obj_particle_spark::hit_something(int offset_after_hit, int offset_before_hit)
{
	/*
	Splitting sparks has been disabled until I can figure out why the number of
  sparks explodes sometimes (hits the 2 << 16 ceiling)
	int split = ROUND(((temp/100)+(sqrt(SQR(xspeed)+SQR(yspeed))/1000))/100);
	if (split) {
		if (split > 4) {
			logmsg(lt_warning,"Split became %i in spark",split);
			split = 0;
			done = true;
		}
		for (int i=0;i<split;i++) {
			Clientworld *clientworld = (Clientworld *)world;

			Obj_particle_spark *part = (Obj_particle_spark *)clientworld->newanonobject(ot_particle_spark);
			if (part) {

				Vector speed;
				speed.x = -xspeed;
				speed.y = -yspeed;
				speed.scale(0.7);
				speed.rotate(PI/2*rnd(-1000,1000)/1000.0);
   	
				Vector off = speed;
				off.normalize();
				off.scale(2);
   	
				part->init(xpos+off.x, ypos+off.y, speed.x, speed.y);
				part->temp = temp;
			} else {
				done = true;
				return;
			}	
		}
		done = true;
		return;
	}
  */
	const int32 *before_pixel = world->map_buffer.get_pixels_read()+offset_before_hit;
	if (!IS_PASSABLE(*before_pixel)) {
  	done = true;
    return;
	}

	if (temp <= 0) {
		int32 *pixel = world->map_buffer.get_pixels_write()+offset_after_hit;

		if (!IS_PASSABLE(*pixel)) {
			*pixel = color | (*pixel & (PF_ALPHA_BITS | PF_TYPE_BITS));
		}

		done = true;
	} else
		bogo_bounce();
}

//=============================================================================
//= Smoke
//=============================================================================
void Obj_particle_smoke::view_draw(int32 *target)
{
/*
	if (age()>0.7 && !has_split && ) {
		//create new smoke particle here...
		has_split = true;
	}
*/
	int alpha = ROUND(15-(15*age()/2));
	if (alpha > 15) alpha = 15;
	
	if (alpha <= 0) {
		done = true;
	} else {
		if (!IS_PASSABLE(*target)) {
			done=true;
			return;
		}

		int32 aa = PIXELA(0,0,0,alpha);
		int32 type = *target & PF_TYPE_BITS;
	
		if (IS_OPAQUE(aa))
			*target = (*target & PF_RGB_BITS) | type;

		else if (IS_INVISIBLE(aa)) {
			//Don't modify the pixel at all
   	
		}	else {
    	//ARGH! ARGH! it's not opaque and it's not invisible, so I gotta do alpha now.
 	  	
			int a = alpha;
			int a1 = 16-alpha;
  		
			int r = (GETRED(color)*a   + GETRED(*target)*a1)   >> 4;
			int g = (GETGREEN(color)*a + GETGREEN(*target)*a1) >> 4;
			int b = (GETBLUE(color)*a  + GETBLUE(*target)*a1)  >> 4;
  		
			*target = PIXEL(r,g,b) | type;
		}
	}
}

//-----------------------------------------------------------------------------
void Obj_particle_smoke::hit_something(int offset_after_hit, int offset_before_hit)
{
	done = true;
}

