/***************************************************************************
                          obj_particle.h  -  description
                             -------------------
    begin                : Thu Nov 4 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#ifndef OBJ_PARTICLE_H
#define OBJ_PARTICLE_H

#include "obj_base.h"

/*
	These are a few of eveyones favorite particle
*/

class Obj_particle : public Obj_base  {
public:

	Obj_particle(World *world_, int id);
	virtual ~Obj_particle();

 	virtual void server_think(){} //This is an anon object so it doesn't make sense to server_think() it

	void init(float x, float y, float xspeed_, float yspeed_);
	virtual void draw(Viewport *viewport);
	virtual void client_think();

	virtual void view_draw(int32 *target) = 0;
	virtual void hit_something(int offset_after_hit, int offset_before_hit) = 0;
protected:
	int32 color;
};

//-----------------------------------------------------------------------------
class Obj_particle_blood : public Obj_particle  {
public:

	Obj_particle_blood(World *world_, int id) : Obj_particle(world_,id) {
		color = mkrndcolor(PIXEL(255,40,40),PIXEL(100,0,0));
	};

	virtual Object_type gettype() {return ot_particle_blood;}

	virtual void view_draw(int32 *target);
	virtual void hit_something(int offset_after_hit, int offset_before_hit);
};

//-----------------------------------------------------------------------------
class Obj_particle_debris : public Obj_particle  {
public:
	Obj_particle_debris(World *world_, int id) : Obj_particle(world_,id) {color = 0;}

	void setcolor(int32 newcolor) {color = newcolor & PF_RGB_BITS;}

	virtual Object_type gettype() {return ot_particle_debris;}

	virtual void view_draw(int32 *target);
	virtual void hit_something(int offset_after_hit, int offset_before_hit);
};

//-----------------------------------------------------------------------------
class Obj_particle_railtrail : public Obj_particle  {
public:
	Obj_particle_railtrail(World *world_, int id) : Obj_particle(world_,id) {
  	yaccel = 0; //float
		color = PIXEL(0,0,255);
	}

	void setcolor(int32 newcolor) {color = newcolor & PF_RGB_BITS;}

	virtual Object_type gettype() {return ot_particle_railtrail;}

	virtual void view_draw(int32 *target);
	virtual void hit_something(int offset_after_hit, int offset_before_hit);
};

//-----------------------------------------------------------------------------
class Obj_particle_spark : public Obj_particle  {
public:
	Obj_particle_spark(World *world_, int id) : Obj_particle(world_,id) {
		temp = rnd(500,1000);
	}

	virtual Object_type gettype() {return ot_particle_spark;}

	virtual void view_draw(int32 *target);
	virtual void hit_something(int offset_after_hit, int offset_before_hit);
	
	float temp;
	float old_age;
};

class Obj_particle_smoke : public Obj_particle  {
public:
	Obj_particle_smoke(World *world_, int id) : Obj_particle(world_,id) {
		yaccel = -0.5; //float
		color = PIXEL(100,100,100);
		air_resistance = DEFAULT_DRAG*5;		
		has_split = false;
	}

	void setcolor(int32 newcolor) {color = newcolor & PF_RGB_BITS;}

	virtual Object_type gettype() {return ot_particle_smoke;}

	virtual void view_draw(int32 *target);
	virtual void hit_something(int offset_after_hit, int offset_before_hit);
	
protected:
	bool has_split;
};


#endif

