/***************************************************************************
                          obj_player.cpp  -  description
                             -------------------
    begin                : Sat Oct 30 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "obj_player.h"
#include "loader.h"
#include "graphics.h"
#include "clientworld.h"
#include "all.h"
#include "systemheaders.h"

//-----------------------------------------------------------------------------
Obj_player_avatar::Obj_player_avatar(World *world_, int id) : Obj_base(world_,id)
{
	cur_walk_frame = 0; 	
	damage = 0;
	damage_speed.x = 0;
	damage_speed.y = 0;
	hook_id = -1;
	player_data = NULL;
}

//-----------------------------------------------------------------------------
Obj_player_avatar::~Obj_player_avatar()
{
 	logmsg(lt_debug,"*** avatar deleted ***");
}

//-----------------------------------------------------------------------------
void Obj_player_avatar::init(int owner, int x, int y)
{
	ownerid = owner;
	xpos = x;
	ypos = y;
	radius = 0; //use the objects own collision test.
	health = MAX_HEALTH;
	damage = 0;
	damage_speed.x = 0;
	damage_speed.y = 0;
	state = PRESPAWNING;
}

//-----------------------------------------------------------------------------
void Obj_player_avatar::serialize(Serializer *serializer)
{
	Obj_base::serialize(serializer);


	Controls tmp_controls;
	if (serializer->isreading()) {
		setup_constants();
		if (controls) {
	  	tmp_controls = *controls;
		}
	}

 	tmp_controls.serialize(serializer);

	//copy the controls if it was retrieved:
	if (!serializer->isreading()) {
		setup_constants();
		if (controls) {
	  	*controls = tmp_controls;
		}
	}


	serializer->rw(cur_walk_frame);
	serializer->rw(dig_now);
	serializer->rw(health);
	serializer->rw(state);
	serializer->rw(animation_sequence);

//	if (state == DIEING) {
		serializer->rw(vaporize);
		serializer->rw(damage);
		serializer->rw(damage_speed.x);
		serializer->rw(damage_speed.y);
//	}
}

//-----------------------------------------------------------------------------
bool Obj_player_avatar::is_passable(int offset)
{
	for (int yc=-5*world->map_buffer.get_xsize(); yc<=0; yc += world->map_buffer.get_xsize())
		if (!IS_PASSABLE(world->map_buffer.get_pixels_read()[yc+offset]))
			return false;

/*
	for (int xc=-1; xc<=1; yc += world->map_buffer.xsize)
		if (!IS_PASSABLE(world->map_buffer.pixels[yc-1+offset]))
			return false;

	*/
	return true;
}

//-----------------------------------------------------------------------------
void Obj_player_avatar::draw(Viewport *viewport)
{
//	viewport->drawline(ROUND(xpos),ROUND(ypos),100,100,0x0ff0080,0);

	setup_constants();
	if (!player_data) return;

	Graphic *sprite = player_data->get_current_frame(animation_sequence);
	if (!sprite) {
		logmsg(lt_error,"Unable to get graphic for skin");
		return;
	}

	//Dont draw anything if the avatar is not alive.
	if (state != LIVING) return;

	int x=ROUND(xpos-viewport->xpos);
	int y=ROUND(ypos-viewport->ypos);

	if (x >= 0 && x < viewport->get_xsize() &&
			y >= 0 && y < viewport->get_ysize()) {

/*
		{
			for (float c=0; c < PI*2; c+=0.2) {
				Vector v = rad2vector(c);
				int xc = 0;
				int yc = 0;
				if (world->collision(ROUND(xpos),       ROUND(ypos-4),
														 ROUND(xpos+v.x*100),ROUND(ypos-4+v.y*100),
														 xc, yc, 0)) {

					viewport->raw_target.pixels[(xc-viewport->xpos)+(yc-viewport->ypos)*viewport->raw_target.xsize] = PIXEL(255,0,255);						
        }
			}
		}
*/

		sprite->draw_view(viewport,x,y);

		int world_owner = ((Clientworld *)world)->getowner();
		if (ownerid == world_owner) {

  		//draw crosshairs
			Vector v = rad2vector(elev2rad(elevation, direction));
			int aimx = ROUND(x+v.x*10);
			int aimy = ROUND(y-4+v.y*10);
			viewport->draw_pixel_opaque(aimx,aimy,PIXEL(255,255,255));
		}
	}
}

//-----------------------------------------------------------------------------
void Obj_player_avatar::server_think()
{
  //This should never happen.
	if (world->isclient()) return;

	setup_constants();
	if (!controls) {
  	logmsg(lt_error,"bitch and moan!");
		return;
	}
	if (!player_data) return;

	//set up a few of our favorite things (good thing World::get_object() is so fast ;)
	Serverworld *serverworld = (Serverworld *)world;

	if (controls->get_flag(Controls::cf_walk))
		animation_sequence = SEQUENCE::WALK;
  else
		animation_sequence = SEQUENCE::STOP;

	//react to the state of the controls.

	//The actual digging is handled by the client code.
	dig_now = controls->get_flag(Controls::cf_dig);

	//jumping
	if (controls->get_flag(Controls::cf_jump)) {
		//only jump if we are on the ground.
		// modify y speed, maybe amplyfy xspeed.
		if (ABS(yspeed) < 10 && canmove() && onground()) {
			yspeed = -45;
			if (controls->get_flag(Controls::cf_walk))
				xspeed = direction*45;
		}

	//walking:
	} else if (controls->get_flag(Controls::cf_walk)) {
		cur_walk_frame++;
		cur_walk_frame &= 3;

		if (onground() && canmove()) {
			bool step_taken = false;
      for (int step=3; step > -4;step--)
//				if (IS_PASSABLE(world->map_buffer.pixels[direction+ROUND(xpos)+ (ROUND(ypos)+step)*world->map_buffer.xsize])) {
				if (
						Obj_base::is_passable(direction+ROUND(xpos),ROUND(ypos)+step) &&
						! Obj_base::is_passable(direction+ROUND(xpos),ROUND(ypos)+step+1)
						) {
					xpos = ROUND(xpos)+direction;
					ypos = ROUND(ypos)+step;
					yspeed = 0;	xspeed = 0;
					step_taken = true;
					break;
				}

			if (!step_taken && Obj_base::is_passable(direction+ROUND(xpos), ROUND(ypos)+4)) {
				xpos = ROUND(xpos)+direction;
				xspeed += direction*5;
			}

			yspeed = 0;

		} else {
			xspeed += direction;
		}

	}

	if (controls->get_flag(Controls::cf_shoot)) {
		Obj_weapon *weapon = player_data->get_weapon(controls->get_weapon());
		if (weapon) {
			if (weapon->canfire()) {
      	weapon->fire(xpos,ypos,elev2rad(elevation,direction));
			} else {
       	logmsg(lt_message,"You pull the trigger, but nothing happens...");
			}			
		}
	}

	if (controls->get_flag(Controls::cf_hook_fire))
		hook_fire();	

	if (controls->get_flag(Controls::cf_hook_release))
		hook_release();	

	if (controls->get_flag(Controls::cf_hook_in))
		hook_in();	

	if (controls->get_flag(Controls::cf_hook_out))
		hook_out();	

	if (state == SPAWNING) {
		xspeed = 0;
		yspeed = 0;
	  state = LIVING;
	}

	if (state == PRESPAWNING) {
		xspeed = 0;
		yspeed = 0;
	  state = SPAWNING;
	}

	if (state == DIEING) {
		logmsg(lt_debug,"avatar has died.");
		//Report our death to the scoreboard:
		Obj_scoreboard *sb= world->getscoreboard();
		if (sb)
			sb->addfrag(hurter_id,ownerid);
		else	
			logmsg(lt_error,"Unable to find the scoreboard for reporting the death of the avatar (this cannot happen)");
		
		done = true;
		return;
	}

	Map_hit hit = update_simple(); //client_think();
	if (hit.hittype != ht_none)
			bogo_bounce();

	//Handle the _PAIN_
	if (damage != 0) {
		logmsg(lt_debug,"avatar hit: health: %.1f, damage: %.1f",health,damage);

		if (health == 0) {
			health = 1;
			logmsg(lt_assert,"This cannot happen! health was 0");
		}

		health -= damage;

		//see if we died.
		if (health <= 0) {
			//calculate how much of the worm will be completely vaporized 0..1		
//			vaporize = damage / ((health+damage)*2);
			vaporize = 0.5;
			if (vaporize > 1) vaporize = 1;

			int gibs = 5;
			for (int c=0;c<gibs;c++) {
  	  	Obj_gib *gib = (Obj_gib *)serverworld->newobject(ot_gib);
				if (gib)
					gib->init(
						xpos,
						ypos,
						xspeed+damage_speed.x+rnd(-20,20),
						yspeed+damage_speed.y+rnd(-20,20),
						player_data->getcolor()
					);
			}

			state = DIEING;
		} else {
			xspeed += KNOCK_BACK*(damage*damage_speed.x);
			yspeed += KNOCK_BACK*(damage*damage_speed.y);	
  	 	damage = 0;
			damage_speed.x = 0;
			damage_speed.y = 0;
		}
	} 		

	setdirty();
}

//-----------------------------------------------------------------------------
void Obj_player_avatar::client_think()
{
	//handle the different states:
	if (state == SPAWNING) {
  	world->makehole(ROUND(xpos),ROUND(ypos),16);
		//init an animation too?

		return;
	}

	if (state == DIEING) {
		if (world->isclient()) {
			Clientworld *clientworld = (Clientworld *)world;

			int blood = 250 + ROUND(vaporize*200*clientworld->gorelevel);

			logmsg(lt_debug,"avatar dieing: %i (%.0f,%.0f)",blood,damage_speed.x,damage_speed.y);

			int hx = ROUND(damage_speed.x*damage);
			if (hx > 100) hx = 100;
			if (hx < -100) hx = -100;

			int hy = ROUND(damage_speed.y*damage);
			if (hy > 100) hy = 100;
			if (hy < -100) hy = -100;
		
	 		for (int c=0;c< blood ;c++) {
				Obj_particle_blood *drop = (Obj_particle_blood *)clientworld->newanonobject(ot_particle_blood);
				if (drop)
					drop->init(xpos+rnd(-2,2),ypos-4+rnd(-4,4),
								xspeed+ROUND(rnd(-100+hx,+100+hx)),
								yspeed+ROUND(rnd(-100+hy,+100+hy))
						);
			}

		}
		world->time_of_death = world->gettime();
 		return;
	}

	//the digging code here is pretty ugly, but be careful touching it,
	//the constants are finely tuned to give smooth digging, altering them will probably upset it.
	if (dig_now) {	

		//find the direction to dig in
		float digelev = controls->get_elevation()-(PI/8);

		//and limit the steepness to something that actually looks good 3/4 pi is 45 deg up.
		if (digelev > 3*PI/4) digelev = 3*PI/4;
	
		//create a vector in the direction that we are digging in.
		Vector v = rad2vector(elev2rad(digelev,direction));

		//create a vector to position the center of the hole with, this is needed so that the bottom
		//of the hole always hits the bottom of the bottom of the worm.
		Vector vr = rot90ccw(v);

		//we need to rotate the vector 90 deg clockwise if we are digging east
		if (direction != -1) vr = rot90ccw(rot90ccw(vr));

		//calc the starting hole center
		float hcx = xpos+vr.x*5+1;
		float hcy = ypos+vr.y*5;
		
		//dig a number of holes in the right direction
		for (float c=0; c < 2; c++)
			world->makehole(ROUND(hcx+c*v.x),ROUND(hcy+c*v.y),12);   	
	}

	//do the strict client side stuff:
	if (world->isclient()) {
		Clientworld *clientworld = (Clientworld *)world;

		float blood = 1-(health/MAX_HEALTH);

		if (blood > 0.3) {
			float pressure = 1-(ROUND(100*age()) % 400)/400.0;
			pressure *= blood;

  		for (int c=0;c< ROUND(pressure*5*clientworld->gorelevel) ;c++) {
				Obj_particle_blood *drop = (Obj_particle_blood *)clientworld->newanonobject(ot_particle_blood);
				if (drop) {
					Vector v;
					v.x = ROUND(rnd(-1-direction*10,1-direction*20));
					v.y = ROUND(rnd(-20,-55));
					v.normalize();	
					v.scale(pressure*80);
		
					drop->init(
							xpos+rnd(-1,1),ypos-4+rnd(-1,1),
							ROUND(xspeed+v.x),ROUND(yspeed+v.y)
						);
				}
			}
		}

	}

	newton_update(world->gettime());
}

#define HOOK_SPEED 1500

//-----------------------------------------------------------------------------
void Obj_player_avatar::hook_fire()
{
	setup_constants();

	//get rid of the old hook.
	hook_release();	

	//Create a new hook if we didn't have one already
	Serverworld *serverworld = (Serverworld *)world;
	Obj_hook *hook = (Obj_hook *)serverworld->newobject(ot_hook);
	hook_id = hook->getid();

	//fire the hook:
	Vector dir = rad2vector(elev2rad(elevation,direction));	
	hook->init(ownerid,ROUND(xpos)+dir.x*5,ROUND(ypos-4)+dir.y*5,xspeed+dir.x*HOOK_SPEED,yspeed+dir.y*HOOK_SPEED);
}

//-----------------------------------------------------------------------------
void Obj_player_avatar::hook_release()
{
  Obj_hook *hook = (Obj_hook *)world->get_object(hook_id,ot_hook);

	if (hook) {
  	hook->bedone();
		hook_id = -1;
	}	
}

//-----------------------------------------------------------------------------
void Obj_player_avatar::hook_in()
{
  Obj_hook *hook = (Obj_hook *)world->get_object(hook_id,ot_hook);
	if (hook) {
  	hook->reel_in();
	}
}

//-----------------------------------------------------------------------------
void Obj_player_avatar::hook_out()
{
  Obj_hook *hook = (Obj_hook *)world->get_object(hook_id,ot_hook);
	if (hook) {
  	hook->reel_out();
	}
}

//-----------------------------------------------------------------------------
void Obj_player_avatar::setup_constants()
{
	if (!player_data)
		player_data = get_owner_data();
	
	if (!player_data) {
		logmsg(lt_fatal,"Unable to get_owner_data() in Obj_player_avatar::setup_constants(), going down!");
		controls = NULL;
		return;
	}
	controls = &player_data->controls;
	direction = controls->get_flag(Controls::cf_direction_east)?1:-1;
	elevation = controls->get_elevation();
}

void Obj_player_avatar::hurt(float hitpoints, Vector speed, int hurter)
{
	world->assert_object_type(hurter,ot_player_data);
	damage += hitpoints;
	damage_speed.add(speed);
	hurter_id = hurter;
}
