/***************************************************************************
                          obj_player_data.cpp  -  description
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "obj_player_data.h"
#include "all.h"
//-----------------------------------------------------------------------------
Obj_player_data::Obj_player_data(World *world_, int id) : Obj_base(world_, id)
{
	strcpy(name,"zombie");
	color = PIXEL(255,128,128);
	avatar_objectid = -1;
	for (int c=0;c<WEAPON_SLOTS;c++)
		weapons[c] = -1;

	animator = NULL;
}

//-----------------------------------------------------------------------------
Obj_player_data::~Obj_player_data()
{
	//find the avatar and kill it!
	Obj_player_avatar *avatar = get_player_avatar();
	if (avatar) {
  	avatar->bedone();
	}	

	for (int slot = 0;slot <WEAPON_SLOTS;slot++) {
		if (weapons[slot] >= 0)	{
  		Obj_weapon *old_weapon = get_weapon(slot);
			if (old_weapon) {
				old_weapon->bedone();
				weapons[slot] = -1;
			}
		}
	}

	if (animator) delete animator;
}

//-----------------------------------------------------------------------------
void Obj_player_data::serialize(Serializer *serializer)
{
	controls.serialize(serializer);
	serializer->rw(color);
	serializer->rw(avatar_objectid);
	for (int c=0;c<WEAPON_SLOTS;c++)
		serializer->rw(weapons[c]);
	serializer->rwstr(name,sizeof(name));
}

//-----------------------------------------------------------------------------
void Obj_player_data::set_weapon(int slot,Object_type object_type)
{
	if (world->isclient()) {
		logmsg(lt_error,"set_weapon called on client");
		return;
	}

	if (slot < 0 || slot >= WEAPON_SLOTS) {
		logmsg(lt_error,"An invalid slot was selected for set_weapon(): %i",slot);
		return;
	}

	//if there is a weapon in the slot then tell it to terminate
	if (weapons[slot] >= 0)	{
  	Obj_weapon *old_weapon = get_weapon(slot);
		if (old_weapon) {
			old_weapon->bedone();
			weapons[slot] = -1;
		}
	}

	//create the new weapon and save its id:
	Serverworld *serverworld = (Serverworld *)world;
 	Obj_weapon *new_weapon = (Obj_weapon *)serverworld->newobject(object_type);
	if (new_weapon) {
		new_weapon->init(objectid);
		weapons[slot] = new_weapon->getid();
	}
}

//-----------------------------------------------------------------------------
Obj_weapon *Obj_player_data::get_weapon(int slot)
{
	if (slot < 0 || slot >= WEAPON_SLOTS) {
		logmsg(lt_error,"An invalid slot was selected for get_weapon(): %i",slot);
		return NULL;
	}
	
	int weapon_objectid = weapons[slot];

	if (weapon_objectid < 0) {
//		logmsg(lt_error,"An empty slot was selected for get_weapon(): %i",slot);
		return NULL;
	} else
		return (Obj_weapon *)world->get_object(weapon_objectid,ot_error);
}

//-----------------------------------------------------------------------------
void Obj_player_data::respawn()
{
	if (world->isclient()) return;
	Serverworld *serverworld = (Serverworld *)world;
	Obj_player_avatar *player_avatar = (Obj_player_avatar *)serverworld->newobject(ot_player_avatar);
	if (!player_avatar) return;

	bool found=false;
	int x;
	int y;
	while (!found) {
		found = true;
  	x = rnd(16,world->map_buffer.get_xsize()-16);	
  	y = rnd(16,world->map_buffer.get_ysize()-16);	
    logmsg(lt_debug,"Trying to spawn the avatar at: %i,%i",x,y);
		for (int ty=0;ty < 16;ty++) {
			for (int tx=0;tx < 16;tx++) {
				int px = x+tx-8;
				int py = y+ty-8;
				const int32 *pixel = world->map_buffer.get_pixels_read()+px+py*world->map_buffer.get_xsize();
      	if (IS_STONE(*pixel)) {
        	found = false;	
				}
			}
		}
	}

	player_avatar->init(objectid,x,y);
	avatar_objectid = player_avatar->getid();
	setdirty();
}
	
//-----------------------------------------------------------------------------
void Obj_player_data::init(Net_joinrequest *joinrequest)
{
	//This object is it's own owner
	ownerid = objectid;

	strcpy(name,joinrequest->name);
	color = joinrequest->color;

//	set_weapon(0,ot_weapon_test);
	set_weapon(0,ot_weapon_bignuke);
	set_weapon(1,ot_weapon_larpa);
	set_weapon(2,ot_weapon_shotgun);
	set_weapon(3,ot_weapon_clusterbomb);
//	set_weapon(3,ot_weapon_plasmagun);
	set_weapon(4,ot_weapon_railgun);
//	set_weapon(1,ot_weapon_minigun);
//	set_weapon(1,ot_weapon_rocket);
//	set_weapon(3,ot_weapon_megashotgun);
	
	//spawn us a new player:
	respawn();
}

//-----------------------------------------------------------------------------
void Obj_player_data::server_think()
{
	Obj_player_avatar *avatar = (Obj_player_avatar *)world->get_object(avatar_objectid,ot_player_avatar);
	
	if (!avatar) {
  	//The avatar has died, respawn
		respawn();
	} else {
  	//ooh we are still alive, lets hope we stay that way...
	}
}

//-----------------------------------------------------------------------------
void Obj_player_data::client_think()
{
}

//-----------------------------------------------------------------------------
Graphic *Obj_player_data::get_current_frame(char sequence)
{
	if (!animator) {
/*
		animation = new Animation;
		animation->create(world->loader,"skins/worm/skin.script",NOCOLOR);
		animator = new Animator;
		animator->create(animation);
*/
		animator = world->loader->animations.new_animator("skins/worm/skin.script",color);
	}

	if (current_sequence != sequence) {
		switch (sequence) {
			case SEQUENCE::INIT: animator->start_sequence("init"); break;
			case SEQUENCE::STOP: animator->start_sequence("stop"); break;
			case SEQUENCE::WALK: animator->start_sequence("walk"); break;
			case SEQUENCE::JUMP: animator->start_sequence("jump"); break;
			case SEQUENCE::LAND: animator->start_sequence("land"); break;
			default: {
				int i=sequence;
      	logmsg(lt_error,"Invalid sequence code passed to Obj_player_data::get_current_frame(): %i",i);
				sequence = -1;
			}
		}
		if (sequence >= 0) current_sequence = sequence;
	}

	Graphic *gfx = animator->get_current_frame(
		world->gettime(),
		controls.get_elevation(),
		controls.get_flag(Controls::cf_direction_east)?1:-1
	);

  if (!gfx) {
		animator->start_sequence("init");
		gfx = animator->get_current_frame(
			world->gettime(),
			controls.get_elevation(),
			controls.get_flag(Controls::cf_direction_east)?1:-1
		);	
	}

	return gfx;
}
