/***************************************************************************
                          obj_rocket.cpp  -  description
                             -------------------
    begin                : Tue Feb 22 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "obj_rocket.h"
#include "all.h"

Obj_rocket::Obj_rocket(World *world_, int id) : Obj_base(world_,id)
{
	for(int i=0; i<8; i++)
		sprite[i]=0;
}
Obj_rocket::~Obj_rocket()
{
}
       
void Obj_rocket::init(int owner, float xpos_, float ypos_, float xspeed_, float yspeed_, float trail_, float power_, int size_)
{
	ownerid = owner;
	xpos = xpos_;
	ypos = ypos_;
	xspeed = xspeed_;
	yspeed = yspeed_;
	yaccel = DEFAULT_ACCEL/3;
	createtime=world->gettime();
	trail=trail_;
	power=power_;
	size=size_;
	last=0;
	air_resistance=0.007;
}

//The frames are numbered like this:
//234
//1 5
//076
void Obj_rocket::draw(Viewport *viewport)
{
	int x = ROUND(xpos)-viewport->xpos;
	int y = ROUND(ypos)-viewport->ypos;
	
	double rad=atan2(yspeed,xspeed); // Calculated the direction
	// rad between PI and -PI, 0 = due right, clockwise +
	
	rad *= 4.0 / PI;
	// now between -4 and 4

	int frame = (int)(rad +5.5);
	frame %= 8;	

	if (!sprite[frame]) {
		// *WARNING* Could this be static??  Same goes for grep STATIC_STRING. email qute@cutey.com if you know the answer
		char s[100];
		sprintf(s,"obj/larpa/larpa%i.rgbf",frame);
		//logmsg(lt_fatal,"fil: %s rad=%f\n  dir %d",s,rad,dir);
		sprite[frame] = world->loader->graphics.getgraphic(s);
		if (!sprite[frame]) return;
	}
	
	sprite[frame]->draw_view(viewport,x,y);
}

void Obj_rocket::serialize(Serializer *serializer)
{
	Obj_base::serialize(serializer);
       
	serializer->rw(trail);
	serializer->rw(power);
	serializer->rw(last);
	serializer->rw(size);
}

void Obj_rocket::server_think()
{
	Serverworld *serverworld = (Serverworld *)world;
       
	Map_hit hit = update_simple(true); //this calls client_think() for us and checks for collisions

	bool blow=false;

	if(trail>0)
		if((age()>0.03) && (age()>last+trail)) {
			last+=trail;
			Obj_projectile *bullet = (Obj_projectile *)serverworld->newobject(ot_projectile);
			if (bullet) bullet->init(
				ownerid,
				xpos,ypos,
				xspeed/10+rnd(-15,15),
				25+yspeed/5+rnd(-1,1),
				100,
				4,
				0,
				0
			);
		}

	if (hit.hittype != ht_none) {
		net_dirty = true;
		blow=true;
	}
       
	if(blow) {
		done=true;
		Obj_explosion *explosion = (Obj_explosion *)serverworld->newobject(ot_explosion);

		if (explosion) explosion->init(
			ownerid,
			xpos,ypos,
			size,
			power
		);

	}
}

