/***************************************************************************
                          obj_scoreboard.cpp  -  description
                             -------------------
    begin                : Sun Dec 26 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "obj_scoreboard.h"
#include "obj_player_data.h"

//-----------------------------------------------------------------------------
Obj_scoreboard::Obj_scoreboard(World *world_, int id) : Obj_base(world_, id)
{
	scores_inuse = 0;
}

//-----------------------------------------------------------------------------
Obj_scoreboard::~Obj_scoreboard()
{
}
	
//-----------------------------------------------------------------------------
void Obj_scoreboard::serialize(Serializer *serializer)
{
	serializer->rw(scores_inuse);
	{ //MS-compiler bug workaround
		for(int i=0;i<scores_inuse;i++) {
			serializer->rw(scores[i].player_id);
			serializer->rw(scores[i].kills);
			serializer->rw(scores[i].deaths);
			serializer->rw(scores[i].frags);
		}
  }
	//put all the names last this is a small hack to get as much out of soft serializing as possible

	{ //MS-compiler bug workaround
		for(int i=0;i<scores_inuse;i++)
			serializer->rwstr(scores[i].playername,sizeof(scores[i].playername));
	}

}

//-----------------------------------------------------------------------------
void Obj_scoreboard::addfrag(int killer_id, int victim_id)
{
	logmsg(lt_debug,"frag: %i %i",killer_id, victim_id);
	Obj_player_data *killer_player = (Obj_player_data *)world->get_object(killer_id,ot_player_data);
	Obj_player_data *victim_player = (Obj_player_data *)world->get_object(victim_id,ot_player_data);

	//First update the killer:
	//If this wasn't a guy killing himself, give the killer a frag
	{	
		int frag = killer_id==victim_id?-1:1;
		int i=get_index(killer_id);
		if (i >= 0) {
			scores[i].kills++;
			scores[i].frags += frag;
			if (killer_player) strcpy(scores[i].playername,killer_player->get_player_name());
		} else {
			if (killer_player)
				strcpy(scores[scores_inuse].playername,killer_player->get_player_name());
			else
				strcpy(scores[scores_inuse].playername,"Unknown.");

			scores[scores_inuse].player_id = killer_id;
			scores[scores_inuse].kills = 1;
			scores[scores_inuse].deaths = 0;  	
			scores[scores_inuse].frags = frag;
			scores_inuse++;
		}
	}

	//Update the victim:
	{
		int i=get_index(victim_id);
		if (i >= 0) {
			scores[i].deaths++;
			if (victim_player) strcpy(scores[i].playername,victim_player->get_player_name());
		} else {
			if (victim_player)
				strcpy(scores[scores_inuse].playername,victim_player->get_player_name());
			else
				strcpy(scores[scores_inuse].playername,"Unknown.");

			scores[scores_inuse].player_id = victim_id;
			scores[scores_inuse].kills = 0;
			scores[scores_inuse].deaths = 1;  	
			scores[scores_inuse].frags = 0;
			scores_inuse++;
		}
	} 	
	setdirty();
}

//-----------------------------------------------------------------------------
Obj_scoreboard::Score Obj_scoreboard::getscore(int index)
{
	if (index >= 0 && index < scores_inuse) {
		return scores[index];
	} else {
		Score s;
		s.player_id = -1;
		s.frags = 0;
		s.kills = 0;
		s.deaths = 0;
		return s;
	}
}
	
//-----------------------------------------------------------------------------
int Obj_scoreboard::get_index(int player_id)
{
	for(int i=0;i<scores_inuse;i++)
		if (scores[i].player_id == player_id)
			return i;
	return -1;
}

//-----------------------------------------------------------------------------
void Obj_scoreboard::dump()
{
	logmsg(lt_message,"%i scores:",scores_inuse);
	for(int i=0;i<scores_inuse;i++){
		Obj_player_data *pd = (Obj_player_data *)world->get_object(scores[i].player_id,ot_player_data);
		if (pd)
  		logmsg(lt_message,"%s: f:%2i k:%2i d:%2i",pd->get_player_name(),scores[i].frags,scores[i].kills,scores[i].deaths);
		else
  		logmsg(lt_message,"%s: f:%2i k:%2i d:%2i",scores[i].playername,scores[i].frags,scores[i].kills,scores[i].deaths);
	}	
}