/***************************************************************************
                          soundsystem.cpp  -  description
                             -------------------
    begin                : Mon Feb 21 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/


#include "fluff.h"
#include "soundsystem.h"
#include "map"
#include "loader.h"

#include "SDL/SDL_mixer.h"


//-------------------------------------------------------------------
class Strcmp { public:
	bool operator()(char* s1, char* s2) const
  {
    return strcmp(s1, s2) < 0;
  }	
};

//-------------------------------------------------------------------
class Sound_loaded {
	public:
	Sound_loaded(char *name, int id, Mix_Chunk *sdl_chunk) {
		sample_name = strdup(name);
		sound_id = id;
		chunk = sdl_chunk;
	}
	~Sound_loaded() {
		free (sample_name);
		Mix_FreeChunk(chunk);
	}

	char *sample_name;
	int sound_id;
	Mix_Chunk *chunk;
};

/*
	Lookup samples by name or by id
*/
class Sounds_loaded_name : public std::map<char *, Sound_loaded*, Strcmp>{};
class Sounds_loaded_id   : public std::map<int   , Sound_loaded*>{};

//-------------------------------------------------------------------
Soundsystem::Soundsystem(Loader *loader_)
{
	dirty = false;
	sounds_loaded_name = new Sounds_loaded_name;
	sounds_loaded_id = new Sounds_loaded_id;
	loader = loader_;

	int frequency;
	Uint16 format;
	quietmode = Mix_QuerySpec(&frequency,&format,&channels);
	if (quietmode) {
		logmsg(lt_debug,"Sound system initialized in quiet mode, we must be on the server then.");
	} else {
		logmsg(lt_debug,"Sound system initialized, mixing %i channels at %iHz.",channels,frequency);
	}
}

//-------------------------------------------------------------------
Soundsystem::~Soundsystem()
{
	Sounds_loaded_name::iterator i=sounds_loaded_name->begin();
  while (i != sounds_loaded_name->end())
		delete (i++)->second;	

	delete sounds_loaded_name;
	delete sounds_loaded_id;
}

void Soundsystem::serialize(Serializer *serializer)
{

}

int Soundsystem::get_sample_id(char *sample_name)
{
	Sounds_loaded_name::iterator i=sounds_loaded_name->find(sample_name);

	if (i != sounds_loaded_name->end())
		return i->second->sound_id;	

	void *sample_ptr = (void *)loader->getfile(sample_name);
	int sample_size = loader->filesize(sample_name);
	logmsg(lt_debug,"Loading sample: %s (%i bytes)",sample_name,sample_size);

	Mix_Chunk *chunk = Mix_LoadWAV_RW(SDL_RWFromMem(sample_ptr, sample_size), 1);

	Sound_loaded *newsound = new Sound_loaded(sample_name, next_sound_id++, chunk);

	sounds_loaded_name->insert(std::pair<char *, Sound_loaded*>(newsound->sample_name,newsound));
	sounds_loaded_id->insert(  std::pair<int, Sound_loaded*>(newsound->sound_id,newsound));

	return newsound->sound_id;
}

/*
This seems to work quite nicely  -lSDL_mixer is needed for this

	Mix_OpenAudio(22100, AUDIO_U8, 1, 1024);

	int channels = Mix_AllocateChannels(48);
	logmsg(lt_debug,"mixer channels: %i",channels);

	void *sample_ptr = loader.getfile("sound/9mm.wav");
	int sample_size = loader.filesize("sound/9mm.wav");
	logmsg(lt_debug,"size of test sample: %i",sample_size);

	Mix_Chunk *chunk = Mix_LoadWAV_RW(SDL_RWFromMem(sample_ptr, sample_size), 1);

	int i = 0;
	while (1) {
		Mix_PlayChannel(-1,chunk,0);
		i = (i+1) % 8;
		logmsg(lt_debug,"%i",i);
	  usleep(100000);
	}
return;
*/
