/***************************************************************************
                          world.cpp  -  description
                             -------------------
    begin                : Sun Oct 10 1999
    copyright            : (C) 1999 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#include "systemheaders.h"
#include "world.h"
#include "fluff.h"
#include "all.h"
#include "world_objects.h"
#include "soundsystem.h"
#include "list"

class Sound_events : public std::list<Sound_event>{};	

//-----------------------------------------------------------------------------------
World::World()
{
	loader = NULL;
	scoreboard_id = -1;
	time_of_death = -1;
	objects = new World_objects;
	next_sound_handle = 1;
	sound_events = new Sound_events;
}

//-----------------------------------------------------------------------------------
World::World(Loader *loader_, Soundsystem *soundsystem_)
{
	sound_events = new Sound_events;
	scoreboard_id = -1;
	time_of_death = -1;
	next_sound_handle = 1;

	objects = new World_objects;
	init(loader_, soundsystem_);
}

//-----------------------------------------------------------------------------------
void World::init(Loader *loader_, Soundsystem *soundsystem_)
{
	loader = loader_;
	soundsystem = soundsystem_;
	scoreboard_id = -1;
	time_of_death = -1;

	for (int c=0; c<16; c++) {
		char gname[100];
		sprintf(gname,"map/hole_%i.rgbf",c+1);
		holes[c] = loader->graphics.getgraphic(gname);
	}
}

//-----------------------------------------------------------------------------------
World::~World()
{
  World_objects::iterator i=objects->begin();
	while (i != objects->end()) {
    delete i->second;
  	i++;
	}
	delete objects;	
	delete sound_events;

//	for (int c=0;c<real_objects_inuse;c++)	delete real_objects[c];
}
/*
//-----------------------------------------------------------------------------------
int World::object_index(int objectid)
{
	bool found = false;
	int low=0;
	int high=real_objects_inuse-1;
	int hit=0;

	while ((!found) && low <= high) {
		hit = (low+high) / 2;
		int k = real_objects[hit]->getid();
		if (k == objectid) found = true;
		else if (k < objectid) low = hit+1;
		else high=hit-1;
	}
	if (found)	
		return hit;
	else
		return -1;
}
*/
//-----------------------------------------------------------------------------------
Obj_base *World::newobject(Object_type objecttype, int objectid)
{
	Obj_base *newobject = NULL;

	switch (objecttype) {
		//the objects that we can create:
		case ot_explosion           : newobject = new Obj_explosion(this, objectid); break;
		case ot_testobject          : newobject = new Obj_test(this, objectid); break;
		case ot_player_avatar       : newobject = new Obj_player_avatar(this, objectid); break;
		case ot_player_data         : newobject = new Obj_player_data(this, objectid); break;
		case ot_gib                 : newobject = new Obj_gib(this, objectid); break;
		case ot_clusterbomb         : newobject = new Obj_clusterbomb(this, objectid); break;
		case ot_grenade             : newobject = new Obj_grenade(this, objectid); break;
		case ot_railslug            : newobject = new Obj_railslug(this, objectid); break;
		case ot_projectile          : newobject = new Obj_projectile(this, objectid); break;
		case ot_hook                : newobject = new Obj_hook(this, objectid); break;
		case ot_scoreboard          : newobject = new Obj_scoreboard(this, objectid); break;
		case ot_plasmaball          : newobject = new Obj_plasmaball(this, objectid); break;
		case ot_fireball            : newobject = new Obj_fireball(this, objectid); break;
		case ot_tinygranade         : newobject = new Obj_tinygranade(this, objectid); break;
		case ot_bignuke             : newobject = new Obj_bignuke(this, objectid); break;
		case ot_rocket              : newobject = new Obj_rocket(this, objectid); break;
		case ot_weapon_larpa        : newobject = new Obj_weapon_larpa(this, objectid); break;
		case ot_weapon_rocket       : newobject = new Obj_weapon_rocket(this, objectid); break;
		case ot_weapon_clusterbomb  : newobject = new Obj_weapon_clusterbomb(this, objectid); break;
		case ot_weapon_mininuke     : newobject = new Obj_weapon_mininuke(this, objectid); break;
		case ot_weapon_bignuke      : newobject = new Obj_weapon_bignuke(this, objectid); break;
		case ot_weapon_test         : newobject = new Obj_weapon_test(this, objectid); break;
		case ot_weapon_minigun		: newobject = new Obj_weapon_minigun(this, objectid); break;
		case ot_weapon_shotgun		: newobject = new Obj_weapon_shotgun(this, objectid); break;
		case ot_weapon_megashotgun  : newobject = new Obj_weapon_megashotgun(this, objectid); break;
		case ot_weapon_railgun      : newobject = new Obj_weapon_railgun(this, objectid); break;
		case ot_weapon_plasmagun    : newobject = new Obj_weapon_plasmagun(this, objectid); break;
		case ot_weapon_flamethrower    : newobject = new Obj_weapon_flamethrower(this, objectid); break;

		//The objects that we cannot:
		case ot_particle_debris:
		case ot_particle_railtrail:
		case ot_particle_spark:
		case ot_particle_smoke:
		case ot_particle_blood	: newobject=NULL; return newobject; logmsg(lt_assert,"someone tried to create an anonymous particle on the server, this is wrong!"); break;
		
		case ot_error : newobject = NULL; return newobject; break;
	}

	objects->insert(std::pair<int,Obj_base*>(objectid,newobject));
//	logmsg(lt_debug,"#%i created: %s",objectid,newobject->getname());

	return newobject;
}

//-----------------------------------------------------------------------------------
void World::initmap(int width, int height)
{
	logmsg(lt_debug,"creating a map that is %i x %i pixels",width,height);

	map_buffer.create(width,height);
	bgmap_buffer.create(width,height);
}

//-----------------------------------------------------------------------------------
//creates a semirandom world.
void World::createmap(int width, int height, unsigned int randomseed_)
{
	randomseed = randomseed_;
	srandom(randomseed);

  //create map
	initmap(width, height);

	logmsg(lt_debug,"populating the map");

	int x, y;
 	for (y=0;y < height;y++)
  	for (x=0;x < width;x++)
 	   	map_buffer.draw_pixel_opaque(x,y,mkrndcolor(0x784008, 0x985414) | PF_TYPE_EARTH);


 	for (y=0;y < height;y++)
 		for (x=0;x < width;x++)
   		bgmap_buffer.draw_pixel_opaque(x,y,mkrndcolor(0x542800, 0x804000) | PF_TYPE_HOLE);

  //place some artifacts:
	Graphic *decor[20];	
//bones:
	decor[0] = loader->graphics.getgraphic("map/bones_1.rgbf");		
	decor[1] = loader->graphics.getgraphic("map/bones_2.rgbf");		
	decor[2] = loader->graphics.getgraphic("map/bones_3.rgbf");		
	decor[3] = loader->graphics.getgraphic("map/bones_4.rgbf");		
//clay:
	decor[4] = loader->graphics.getgraphic("map/clay_1.rgbf");		
	decor[5] = loader->graphics.getgraphic("map/clay_2.rgbf");		
	decor[6] = loader->graphics.getgraphic("map/clay_3.rgbf");		
	decor[7] = loader->graphics.getgraphic("map/clay_4.rgbf");		

//stones:
	decor[11] = loader->graphics.getgraphic("map/stone_1.rgbf");		
	decor[12] = loader->graphics.getgraphic("map/stone_2.rgbf");		
	decor[13] = loader->graphics.getgraphic("map/stone_3.rgbf");		
	decor[14] = loader->graphics.getgraphic("map/stone_4.rgbf");		
	decor[15] = loader->graphics.getgraphic("map/stone_5.rgbf");		
	decor[16] = loader->graphics.getgraphic("map/stone_5.rgbf");		

	//dig a tunnel:
	int c;
	for (c=0; c < 150; c++) holes[12]->draw_map(&map_buffer,&bgmap_buffer,100+c,100);	

	//put the decor:
	for (c=0; c < 30; c++) decor[rnd(11,16)]->draw_map(&map_buffer,&bgmap_buffer,rnd(8,width-8),rnd(8,height-8));	
	for (c=0; c < 50; c++) holes[rnd(0,15)]->draw_map(&map_buffer,&bgmap_buffer,rnd(8,width-8),rnd(8,height-8));	
	for (c=0; c < 50; c++) decor[rnd(4,7)  ]->draw_map(&map_buffer,&bgmap_buffer,rnd(8,width-8),rnd(8,height-8));	
	for (c=0; c < 30; c++) decor[rnd(0,3)  ]->draw_map(&map_buffer,&bgmap_buffer,rnd(8,width-8),rnd(8,height-8));	

	//Frame the map with stone:
	for (x=0; x < map_buffer.get_xsize(); x++)
 		map_buffer.modify_pixel_type(x, 0, PF_TYPE_STONE);

	for (x=0; x < map_buffer.get_xsize(); x++)
 		map_buffer.modify_pixel_type(x, map_buffer.get_ysize()-1, PF_TYPE_STONE);

	for (y=0; y < map_buffer.get_ysize(); y++) {
  	map_buffer.modify_pixel_type(0,y,PF_TYPE_STONE);
  	map_buffer.modify_pixel_type(map_buffer.get_xsize()-1,y, PF_TYPE_STONE);
	}

	logmsg(lt_debug,"Done creating map");
}

//The bits allocated for error accumulation
#define ERRORBITS 16

bool World::collision(int x0, int y0, int x1, int y1, int &xc, int &yc, int radius)
{
	//This routine is a highly perverted bresenham line algorithm
	//it has been optimized in various non-logical ways,
	//be very careful when fooling with it, a slight imperfection will cause havoc.
	//note that the y component is scaled by xsize in the inner loops,
	//so we must add or substract in xsize increments.

	//clamp in the y direction (but not in x, overflowing in x will not segfault:)
	if (y1 > map_buffer.get_ysize()-1) y1 = map_buffer.get_ysize()-1;
	if (y0 > map_buffer.get_ysize()-1) y0 = map_buffer.get_ysize()-1;
	if (y1 < 0) y1 = 0;
	if (y0 < 0) y0 = 0;

	//get delta values
	int dx = x1-x0;
	int dy = y1-y0;

	//zero length is boring (and a dangerous special case) so get rid of it.
	if (dx == 0 && dy == 0) {
   	if (!IS_PASSABLE(map_buffer.get_pixels_read()[x0+y0*map_buffer.get_xsize()])) {
			xc = x0;
			yc = y0;
    	return true;	
		}	else return false;			
	}

	const int32 *pixels = map_buffer.get_pixels_read();
	if (ABS(dx) > ABS(dy)) {
		// x major
	
		int ymove = map_buffer.get_xsize()*SIGN(dy);	//The step to take with y when error hits 1
		dy = (ABS(dy) << ERRORBITS)/ABS(dx);		//The value to add to error each x
		int e = 0;															//error, fixed point with errorbits bits between 0 and 1.
		int x = x0;															//x is just the x coordinate
		int y = y0*map_buffer.get_xsize();			//y is offset into buffer, not the y coordinate (this saves many muls)
		radius *= map_buffer.get_xsize();				//radius is used for y movement, so it is scaled to y
		dx = SIGN(dx);													//we just need the sign of dx
		int old_y = y;

		x1 += dx;
		while (x != x1) {

			//calc the offset to the most interesting pixel
			int offset = x+y; //saves radius*2 adds.

			//center pixel
			if (!IS_PASSABLE(pixels[offset])) {
				xc = x-dx;
				yc = old_y/map_buffer.get_xsize();
				return true;
			}

			//side pixels
			for (int c=map_buffer.get_xsize();c<radius;c+=map_buffer.get_xsize()) {

				if (!IS_PASSABLE(pixels[offset+c]) || !IS_PASSABLE(pixels[offset-c])) {
					xc = x-dx;
					yc = old_y/map_buffer.get_xsize();
					return true;
				}

			}

			//tally the error and ajust
			e += dy;
			if (e >= (1 << ERRORBITS)) {
				old_y = y;
				y += ymove;
				e -= (1 << ERRORBITS);
			}

			//move x one pixel in the right direction
			x += dx;

		} //while (x != x1) {
	
	} else {
		//y major

		int xmove = SIGN(dx);									//The step to take with x when error hits 1 << ERRORBITS
		dx = (ABS(dx) << ERRORBITS)/ABS(dy);	//The value to add to error each y
		int e = 0;														//error, fixed point with ERRORBITS bits between 0 and 1.
		int x = x0;														//x is just the x coordinate
		int y = y0*map_buffer.get_xsize();		//y is offset into buffer, not the y coordinate (this saves many muls)
		int sign_dy = SIGN(dy);
		dy = map_buffer.get_xsize()*sign_dy;	//we just need the sign of dy scaled to one pixel up or down
		y1 = (y1+sign_dy)*map_buffer.get_xsize();
		int old_x = x;


		while (y != y1) {

			//calc the offset to the most interesting pixel
			int offset = x+y; //saves radius*2 adds.

			//center pixel
			if (!IS_PASSABLE(pixels[offset])) {
				xc = old_x;
				yc = (y-dy)/map_buffer.get_xsize();
				return true;
			}

			//side pixels
			for (int c=1;c<radius;c++) {

				if (!IS_PASSABLE(pixels[offset+c]) || !IS_PASSABLE(pixels[offset-c])) {
					xc = old_x;
					yc = (y-dy)/map_buffer.get_xsize();
					return true;
				}

			}

			//tally the error and ajust
			e += dx;
			if (e >= (1 << ERRORBITS)) {
				old_x = x;
				x += xmove;
				e -= (1 << ERRORBITS);
			}

			//move y one pixel in the right direction
			y += dy;

		} //while (y != y1) {

	}

	return false;
}

bool World::collision(int x0, int y0, int x1, int y1, int &xc, int &yc, Obj_base *object)
{
	//This routine is a highly perverted bresenham line algorithm
	//it has been optimized in various non-logical ways,
	//be very careful when fooling with it, a slight imperfection will cause havoc.
	//note that the y component is scaled by xsize in the inner loops,
	//so we must add or substract in xsize increments.

	//This works, so don't fuck with it!

	//get delta values
	int dx = x1-x0;
	int dy = y1-y0;

	//zero length is boring (and a dangerous special case) so get rid of it.
	if (dx == 0 && dy == 0) {
   	if (!object->is_passable(x0,y0)) {
			xc = x0;
			yc = y0;
    	return true;	
		}	else
			return false;			
	}

	if (ABS(dx) > ABS(dy)) {
		// x major
	
		int ymove = map_buffer.get_xsize()*SIGN(dy);	//The step to take with y when error hits 1
		dy = (ABS(dy) << ERRORBITS)/ABS(dx);		//The value to add to error each x
		int e = 0;															//error, fixed point with errorbits bits between 0 and 1.
		int x = x0;															//x is just the x coordinate
		int y = y0*map_buffer.get_xsize();			//y is offset into buffer, not the y coordinate (this saves many muls)
		dx = SIGN(dx);													//we just need the sign of dx
		int old_y = y;

		x1 += dx;
		while (x != x1) {

			//calc the offset to the most interesting pixel
			int offset = x+y; //saves radius*2 adds.

			//Ask the object if it likes the position
			if (!object->is_passable(offset)) {
				xc = x-dx;
				yc = old_y/map_buffer.get_xsize();
				return true;
			}

			//tally the error and ajust
			e += dy;
			if (e >= (1 << ERRORBITS)) {
				old_y = y;
				y += ymove;
				e -= (1 << ERRORBITS);
			}

			//move x one pixel in the right direction
			x += dx;

		} //while (x != x1) {
	
	} else {
		//y major

		int xmove = SIGN(dx);									//The step to take with x when error hits 1 << ERRORBITS
		dx = (ABS(dx) << ERRORBITS)/ABS(dy);	//The value to add to error each y
		int e = 0;														//error, fixed point with ERRORBITS bits between 0 and 1.
		int x = x0;														//x is just the x coordinate
		int y = y0*map_buffer.get_xsize();		//y is offset into buffer, not the y coordinate (this saves many muls)
		int sign_dy = SIGN(dy);
		dy = map_buffer.get_xsize()*sign_dy;	//we just need the sign of dy scaled to one pixel up or down
		y1 = (y1+sign_dy)*map_buffer.get_xsize();
		int old_x = x;


		while (y != y1) {

			//calc the offset to the most interesting pixel
			int offset = x+y; //saves radius*2 adds.

			//Ask the object if it likes the position
			if (!object->is_passable(offset)) {
				xc = old_x;
				yc = (y-dy)/map_buffer.get_xsize();
				return true;
			}

			//tally the error and ajust
			e += dx;
			if (e >= (1 << ERRORBITS)) {
				old_x = x;
				x += xmove;
				e -= (1 << ERRORBITS);
			}

			//move y one pixel in the right direction
			y += dy;

		} //while (y != y1) {

	}

	return false;
}

Obj_player_avatar *World::get_player_avatar(int playerid)
{
	Obj_player_data *player_data = (Obj_player_data *)get_object(playerid,ot_player_data);
	if (!player_data) {
		logmsg(lt_error,"Unable to get the Obj_player_data in World::get_player_avatar()");
		return NULL;
	}

	Obj_player_avatar *player_avatar = (Obj_player_avatar *)get_object(player_data->getavatarid(),ot_player_avatar);
	if (!player_avatar) {
		logmsg(lt_warning,"Unable to get the Obj_player_avatar in World::get_player_avatar() %i",player_data->getavatarid());
		return NULL;
	}

	return player_avatar;
}


bool World::get_player_pos(int playerid, int &x, int &y)
{
	Obj_player_avatar *player_avatar = get_player_avatar(playerid);
	if (!player_avatar) {
		logmsg(lt_warning,"Unable to get the Obj_player_avatar in World::get_player_pos()");
		return false;
	}

	float xf=0;
	float yf=0;
	player_avatar->getpos(xf, yf);
	x = ROUND(xf);
	y = ROUND(yf);
	return true;
}

Obj_base *World::get_object(int objectid, Object_type expected_type)
{
  World_objects::iterator i=objects->find(objectid);
	if (i == objects->end()) {
  	return NULL;
	}

	//reality check: (if the expected type is ot_error ignore the type check)
	if (i->second->gettype() == expected_type || expected_type == ot_error)
    return i->second;
	else
		return NULL;
}

void World::assert_object_type(int objectid, Object_type expected_type)
{
	if (!do_assertions()) return;

  World_objects::iterator i=objects->find(objectid);
	if (i == objects->end()) {
  	logmsg(lt_assert,"The object %i was supposed to exist and be a %s (it didn't exist)",objectid,OBJECT_NAMES[expected_type]);
		return;
	}

	if (i->second->gettype() != expected_type)
  	logmsg(lt_assert,"The object %i was supposed to exist and be a %s (it was a %s instead)",objectid,OBJECT_NAMES[expected_type],OBJECT_NAMES[i->second->gettype()]);
}

void World::dump_objects()
{
	logmsg(lt_debug,"Number of objects: %i",objects->size());
  World_objects::iterator i=objects->begin();
	while (i != objects->end()) {
		char owner_name[80];
		Obj_player_data *owner = (Obj_player_data *)i->second->get_owner_data();
		if (owner)
			strcpy(owner_name,owner->get_player_name());
		else
			sprintf(owner_name," %i ?",i->second->getowner());		
	
		logmsg(lt_debug,"%i: owner: %s : %s",i->first,owner_name,i->second->getname());
  	i++;
	}
}

void World::remove_dead_objects()
{
	World_objects::iterator i = objects->begin();
	while (i != objects->end()) {
		if (i->second->isdone()) {
			World_objects::iterator j = i++;
			delete j->second; 	//delete payload
			objects->erase(j);  //delete the item itself
		} else
			i++;
	}
}

int World::getobjcount()
{
	return objects->size();
}

int World::get_player_count()
{
	int count = 0;
	
	World_objects::iterator i = objects->begin();
	while ( i != objects->end() ) {
		if ( i->second->gettype() == ot_player_avatar ) count ++;
		i++;
	}
	
	return count;
}


Obj_player_avatar *World::get_player_number( int iIndex )
{
	int count = -1;
	
	World_objects::iterator i = objects->begin();
	while ( i != objects->end() ) {
		if ( i->second->gettype() == ot_player_avatar ) count ++;
		
		if ( count == iIndex ) return (Obj_player_avatar *)i->second;
		
		i++;
	}
	
	return NULL;
/*
	ot_player_avatar
	
	
	World_objects::iterator i = objects->begin();
	for ( int i=0; i<iIndex; i++ ) {
		if ( i == objects->end() ) return -1;
		iIndex --;
	}
	
	return i->first
		
		if (i->second->isdone()) {
			World_objects::iterator j = i++;
			delete j->second; 	//delete payload
			objects->erase(j);  //delete the item itself
		} else
			i++;
	}*/
}

/*-----------------------------------------------------------------------------
	Sound system stuff:
*/

/*
	enum Sound_flags {
  	SOUND_NORMAL     = 0,
		SOUND_LOOPING    = 1
	};
*/


int World::start_sound(Sound_event &sound_event)
{
	sound_event.sound_handle = next_sound_handle++;
	sound_events->push_front(sound_event);
	return sound_event.sound_handle;
}

int World::start_sound(char *sample_name, int flags, float volume)
{
	//set up the right kind of sound_event, call start_sound() with it
	return 0;
}

int World::start_sound(char *sample_name, int flags, float volume, float x_pos, float y_pos)
{
	//set up the right kind of sound_event, call start_sound() with it
	return 0;
}

int World::start_sound(char *sample_name, int flags, float volume, int object_id)
{
	//set up the right kind of sound_event, call start_sound() with it
	return 0;
}

bool World::stop_sound(int sound_handle)
{
	//set up the right kind of sound_event, call start_sound() with it
	return 0;
}
