/***************************************************************************
                          cmdline.cpp  -  description
                             -------------------
    begin                : Sat Jan 1 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmdline.h"
#include "stdio.h"
#include "string.h"

Cmdline::Cmdline()
{
	topng = true;
	rgba[0] = 0;
	mask[0] = 0;
	rgbf[0] = 0;
}

Cmdline::~Cmdline()
{
}

void Cmdline::complain(char *reason)
{
	printf(reason);
	printf("\nsyntax:\n");
	printf(" rgbf frompng rgba=source_rgba.png [mask=source_mask.png] [rgbf=target.rgbf]\n");
	printf(" rgbf topng   rgbf=source.rgbf [mask=target_mask.png] [rgba=target_rgba.png]\n");
}

void change_extention(char *filename, char *extention)
{
	char *ext=NULL;

	for(char *ch=filename;*ch;ch++)
		if (*ch == '.')
			ext = ch;

	if (ext)
		strcpy(ext,extention);
}

int Cmdline::interpret(int argc, char *argv[])
{
	for (int c=1;c<argc;c++) {
		if (strcmp(argv[c],"topng") == 0)
			topng = true;
		else if (strcmp(argv[c],"frompng") == 0)
			topng = false;

		else if (strncmp(argv[c],"rgba=",5) == 0) { //-- rgba= -----------------

			char *tmp=argv[c]+5;
			if (strlen(tmp) > 0) {
      	strcpy(rgba,tmp);		
			}	else {
				complain("Please tell me the what rgba file to use (or leave out the rgba= option).");    	
				return 0;
			}

		}	else if (strncmp(argv[c],"mask=",5) == 0) { //-- mask= -----------------

			char *tmp=argv[c]+5;
			if (strlen(tmp) > 0) {
      	strcpy(mask,tmp);		
			}	else {
				complain("Please tell me the what mask file to use (or leave out the mask= option).");    	
				return 0;
			}

		}	else if (strncmp(argv[c],"rgbf=",5) == 0) { //-- rgbf= -----------------

			char *tmp=argv[c]+5;
			if (strlen(tmp) > 0) {
      	strcpy(rgbf,tmp);		
			}	else {
				complain("Please tell me the what rgbf file to use (or leave out the rgbf= option).");    	
				return 0;
			}

		} else { //-- Bail out -------------------------------------------------------

			char tmp[200];
			sprintf(tmp,"The option \"%s\" was not understood.",argv[c]);
			complain(tmp);    	
			return 0;
		}
	}

	//Fixup missing options:
	if (topng) {
		if (strlen(rgbf) == 0) {
    	complain("You must give me an input rgbf file if you want it converted to png");
			return 0;
		}
	
		if (strlen(mask) == 0) {
			strcpy(mask,rgbf);
			change_extention(mask,"_mask.png");
		}

		if (strlen(rgba) == 0) {
			strcpy(rgba,rgbf);
			change_extention(rgba,".png");
		}

	} else {

		if (strlen(rgba) == 0) {
    	complain("You must give me an input rgba (png) file if you want it converted to rgbf");
			return 0;
		}
	
		if (strlen(rgbf) == 0) {
			strcpy(rgbf,rgba);
			change_extention(rgbf,".rgbf");
		}
	}

	printf("options:\n");
	if (topng)
		printf(" converting to png from rgbf\n");
	else
		printf(" converting to rgbf from png\n");
	printf(" rgbf=%s\n",rgbf);
	printf(" rgba=%s\n",rgba);
	printf(" mask=%s\n",mask);

	return 1;
}



