/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Sat Jan  1 16:41:50 CET 2000
    copyright            : (C) 2000 by Flemming Frandsen
    email                : dion@swamp.dk
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include "cmdline.h"
#include <ptc/titan.h>

//#include "../../nil/nil/fluff.h"
#include "raw_surface.h"

int topng(Cmdline *cmdline)
{
	Raw_surface g;
	if (!g.load_from_file(cmdline->rgbf)) {
  	printf("Unable to read %s so it's a bit hard for me to continue, bailing out.",cmdline->rgbf);
		return 100;
	}

	Surface rgbf(g.get_xsize(),g.get_ysize(),PIXELFORMAT);
	int32 *rgbf_buffer = (int32 *)rgbf.lock();
	memcpy(rgbf_buffer,g.get_pixels(),g.get_xsize()*g.get_ysize()*sizeof(int32));

	//The saver
	Image *saver = new Image();

	//save the rgba part:
	{
		Surface rgba(g.get_xsize(),g.get_ysize(),PIXELFORMAT);
		int32 *rgba_buffer = (int32 *)rgba.lock();
	
		for(int y=0;y<g.get_ysize();y++)
			for(int x=0;x<g.get_xsize();x++) {
				int32 i = rgbf_buffer[x+y*g.get_xsize()];

				rgba_buffer[x+y*g.get_xsize()] = (i & 0xf0ffffff) | ((i >> 4) & 0x0f000000);
			}

		rgba.unlock();
	 	saver->save(cmdline->rgba, rgba, PNG, NULL);
	}

	//save the mask part:
	{
		Surface mask(g.get_xsize(),g.get_ysize(),PIXELFORMAT);
		int32 *mask_buffer = (int32 *)mask.lock();
	
		for(int y=0;y<g.get_ysize();y++)
			for(int x=0;x<g.get_xsize();x++) {
				int32 flags = rgbf_buffer[x+y*g.get_xsize()] & PF_TYPE_BITS;

				int mask_pixel=0xff000000;

				if (flags == PF_TYPE_VOID)				mask_pixel |= 0x00000000;
				else if (flags == PF_TYPE_HOLE)		mask_pixel |= 0x0000ff00;
				else if (flags == PF_TYPE_EARTH)	mask_pixel |= 0x00ff0000;
				else if (flags == PF_TYPE_STONE) 	mask_pixel |= 0x000000ff;

				mask_buffer[x+y*g.get_xsize()] = mask_pixel;
			}

		mask.unlock();
	 	saver->save(cmdline->mask, mask, PNG, NULL);
	}

	//clean up
	delete saver;
  rgbf.unlock();

	return 0;	
}

int frompng(Cmdline *cmdline)
{
	Image *loader = new Image();

	//load the rgba part:
	loader->load(cmdline->rgba);

	//set up the rgbf surface
	Surface rgbf(loader->width(),loader->height(),PIXELFORMAT);
	int32 *rgbf_buffer = (int32 *)rgbf.lock();

	
  Surface rgba(loader->width(),loader->height(),PIXELFORMAT);
  loader->copy(rgba);
	int32 *rgba_buffer = (int32 *)rgba.lock();

	for(int y=0;y<rgbf.height();y++)
		for(int x=0;x<rgbf.width();x++) {

			int32 i = rgba_buffer[x+y*rgbf.width()];
			int32 auto_type = (i & PF_ALPHA_BITS)?PF_TYPE_EARTH:PF_TYPE_VOID;
			rgbf_buffer[x+y*rgbf.width()] = (i & (PF_ALPHA_BITS|PF_RGB_BITS)) | auto_type;

		}

	rgba.unlock();


	//load the mask if any:
	if (strlen(cmdline->mask) != 0) {
		loader->load(cmdline->rgba);

		if ((unsigned)rgbf.height() != loader->height() || (unsigned)rgbf.width() != loader->width()) {
    	printf("Error: the mask image was %ix%i, but the rgba image was %ix%i\n",loader->width(),loader->height(),rgbf.width(),rgbf.height());
			return 201;
		}

	  Surface mask(loader->width(),loader->height(),PIXELFORMAT);
  	loader->copy(mask);
		int32 *mask_buffer = (int32 *)mask.lock();

		for(int y=0;y<mask.height();y++)
			for(int x=0;x<mask.width();x++) {

				int32 p = mask_buffer[x+y*mask.width()];

				bool redish = GETRED(p) > 128;
				bool greenish = GETGREEN(p) > 128;
				bool blueish = GETBLUE(p) > 128;

				int32 type = PF_TYPE_VOID;

				     if (redish) type = PF_TYPE_EARTH;
				else if (greenish) type = PF_TYPE_HOLE;
				else if (blueish) type = PF_TYPE_STONE;

				rgbf_buffer[x+y*mask.width()] = (rgbf_buffer[x+y*mask.width()] & (PF_ALPHA_BITS|PF_RGB_BITS)) | type;
			}

		mask.unlock();
	}

	//clean up
	delete loader;

	Raw_surface g;
	g.create(rgbf.width(),rgbf.height());
	memcpy(g.get_pixels(),rgbf_buffer,g.get_xsize()*g.get_ysize()*sizeof(int32));

	if (!g.save_to_file(cmdline->rgbf)) {
  	printf("Unable to save to %s so it's a bit hard for me to continue, bailing out.",cmdline->rgbf);
		return 202;
	}

	return 0;	
}

int main(int argc, char *argv[])
{
  printf("rgbf 0.1, this program will convert to and from the rgbf format used by NiL\n\n");

	Cmdline cmdline;
	if (!cmdline.interpret(argc, argv)) return 1;

	if (cmdline.topng)
		return topng(&cmdline);
	else
		return frompng(&cmdline);

	return 0;
}






